@@ -1,7 +1,7 @@
 /*
  * libbrlapi - A library providing access to braille terminals for applications.
  *
- * Copyright (C) 2002-2004 by
+ * Copyright (C) 2002-2005 by
  *   Samuel Thibault <Samuel.Thibault@ens-lyon.org>
  *   Sbastien Hinderer <Sebastien.Hinderer@ens-lyon.org>
  * All rights reserved.
@@ -21,35 +21,53 @@
 
 /* api_client.c handles connection with BrlApi */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <stddef.h>
-#include <string.h>
 #include <inttypes.h>
+#include <string.h>
 #include <errno.h>
-#include <sys/types.h>
-#include <sys/mman.h>
-#include <pthread.h>
-#include <syslog.h>
 #include <fcntl.h>
 #include <sys/stat.h>
+#include <wchar.h>
+#ifndef WINDOWS
+#include <langinfo.h>
+#endif /* WINDOWS */
+#include <locale.h>
+
+#ifdef WINDOWS
+#include <ws2tcpip.h>
+
+#ifdef __MINGW32__
+#include "win_pthread.h"
+#endif /* __MINGW32__ */
+
+#include "misc.h"
+
+#define syslog(level,fmt,...) fprintf(stderr,#level ": " fmt, __VA_ARGS__)
+
+#else /* WINDOWS */
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
 
+#include <pthread.h>
+#include <syslog.h>
+
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #else /* HAVE_SYS_SELECT_H */
 #include <sys/time.h>
 #endif /* HAVE_SYS_SELECT_H */
 
+#define setSocketErrno()
+#endif /* WINDOWS */
+
 #ifdef HAVE_ALLOCA_H
 #include <alloca.h>
 #endif /* HAVE_ALLOCA_H */
@@ -71,6 +89,8 @@
 #include "api.h"
 #include "api_protocol.h"
 #include "brldefs.h"
+
+#define BRLAPI(fun) brlapi_ ## fun
 #include "api_common.h"
 
 #ifndef MIN
@@ -86,40 +106,49 @@
 #define STRAW 2
 #define STCONTROLLINGTTY 4
 
-/* for remembering getaddrinfo error code */
-static int gai_error;
+#ifdef WINDOWS
+static WSADATA wsadata;
+#endif /* WINDOWS */
 
 /* Some useful global variables */
 static unsigned int brlx = 0;
 static unsigned int brly = 0;
 static int fd = -1; /* Descriptor of the socket connected to BrlApi */
-static int truetty = -1;
 
 pthread_mutex_t brlapi_fd_mutex = PTHREAD_MUTEX_INITIALIZER; /* to protect concurrent fd access */
 static int state = 0;
 static pthread_mutex_t stateMutex = PTHREAD_MUTEX_INITIALIZER;
 
+/** key presses buffer size
+ *
+ * key presses won't be lost provided no more than BRL_KEYBUF_SIZE key presses
+ * are done between two calls to brlapi_read* if a call to another function is
+ * done in the meanwhile (which needs somewhere to put them before being able
+ * to get responses from the server)
+ */
+#define BRL_KEYBUF_SIZE 256
+
 /* key presses buffer, for when key presses are received instead of
  * acknowledgements for instance
  *
  * every function must hence be able to read at least sizeof(brl_keycode_t) */
-
+ 
 static brl_keycode_t keybuf[BRL_KEYBUF_SIZE];
 static unsigned keybuf_next;
 static unsigned keybuf_nb;
 static pthread_mutex_t keybuf_mutex = PTHREAD_MUTEX_INITIALIZER; /* to protect concurrent key buffering */
 
-static brlapi_errorHandler_t brlapi_errorHandler = brlapi_defaultErrorHandler;
-static pthread_mutex_t brlapi_errorHandler_mutex = PTHREAD_MUTEX_INITIALIZER;
+static brlapi_exceptionHandler_t brlapi_exceptionHandler = brlapi_defaultExceptionHandler;
+static pthread_mutex_t brlapi_exceptionHandler_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 static void handle_keycode(brl_keycode_t code)
 {
   pthread_mutex_lock(&keybuf_mutex);
   if (keybuf_nb>=BRL_KEYBUF_SIZE) {
     pthread_mutex_unlock(&keybuf_mutex);
-    syslog(LOG_WARNING,"lost key 0x%x !\n",code);
+    syslog(LOG_WARNING,"lost key: 0X%" PRIX32,code);
   } else {
-    keybuf[(keybuf_next+keybuf_nb++)%BRL_KEYBUF_SIZE]=ntohl(code);
+    keybuf[(keybuf_next+keybuf_nb++)%BRL_KEYBUF_SIZE]=code;
     pthread_mutex_unlock(&keybuf_mutex);
   }
 }
@@ -128,7 +157,7 @@ static void handle_keycode(brl_keycode_t code)
 /* Waits for the specified type of packet: must be called with brlapi_fd_mutex locked */
 /* If the right packet type arrives, returns its size */
 /* Returns -1 if a non-fatal error is encountered */
-/* Calls the error handler if an exception is encountered */
+/* Calls the exception handler if an exception is encountered */
 static ssize_t brlapi_waitForPacket(brl_type_t expectedPacketType, void *packet, size_t size)
 {
   unsigned char *localPacket[BRLAPI_MAXPACKETSIZE];
@@ -138,44 +167,45 @@ static ssize_t brlapi_waitForPacket(brl_type_t expectedPacketType, void *packet,
   brl_keycode_t *code = (brl_keycode_t *) localPacket;
   errorPacket_t *errorPacket = (errorPacket_t *) localPacket;
   int hdrSize = sizeof(errorPacket->code)+sizeof(errorPacket->type);
-  while ((res=brlapi_readPacket(fd,&type,localPacket,sizeof(localPacket)))>=0) {
-    if (type==expectedPacketType) {
-      if (res<=size) {
-        if (res>0) memcpy(packet, localPacket, res);
-        return res;
-      } else {
-        brlapi_libcerrno=EFBIG;
-        brlapi_libcerrfun="brlapi_waitForPacket";
-        brlapi_errno=BRLERR_LIBCERR;
-        return -1;
-      }
-    }
-    if (type==BRLPACKET_ERROR) {
-      brlapi_errno = ntohl(*code);
-      return -1;
-    }
-    if (type==BRLPACKET_EXCEPTION) {
-      size_t esize;
-      if (res<hdrSize) esize = 0; else esize = res-hdrSize;
-      brlapi_errorHandler(ntohl(errorPacket->code), ntohl(errorPacket->type), &errorPacket->packet, esize);
-      continue;
-    }
-    pthread_mutex_lock(&stateMutex);
-    st = state;
-    pthread_mutex_unlock(&stateMutex);
-    if ((type==BRLPACKET_KEY) && (st & STCONTROLLINGTTY) && (res==sizeof(brl_keycode_t))) {
-      handle_keycode(ntohl(*code));
-      continue;
-    }
-    syslog(LOG_ERR,"(brlapi_waitForPacket) Received unexpected packet of type %s and size %d\n",brlapi_packetType(type),res);
+
+readNextPacket:
+  res = brlapi_readPacketHeader(fd, &type);
+  if (res<0) goto out;
+  if (type==expectedPacketType) {
+    res = brlapi_readPacketContent(fd,res, packet, size);
+    if (res<0) goto out;
+    return res;
   }
+  
+  if ((res = brlapi_readPacketContent(fd, res, localPacket, sizeof(localPacket)))<0) goto out;
+  if (type==BRLPACKET_ERROR) {
+    brlapi_errno = ntohl(*code);
+    return -1;
+  }
+  if (type==BRLPACKET_EXCEPTION) {
+    size_t esize;
+    if (res<hdrSize) esize = 0; else esize = res-hdrSize;
+    brlapi_exceptionHandler(ntohl(errorPacket->code), ntohl(errorPacket->type), &errorPacket->packet, esize);
+    goto readNextPacket;
+  }
+  pthread_mutex_lock(&stateMutex);
+  st = state;
+  pthread_mutex_unlock(&stateMutex);
+  if ((type==BRLPACKET_KEY) && (st & STCONTROLLINGTTY) && (res==sizeof(brl_keycode_t))) {
+    handle_keycode(ntohl(*code));
+    goto readNextPacket;
+  }
+  syslog(LOG_ERR,"(brlapi_waitForPacket) Received unexpected packet of type %s and size %ld\n",brlapi_packetType(type),(long)res);
+  goto readNextPacket;
+
+out:
   if (res==-2) brlapi_errno=BRLERR_EOF;
   return -1;
 }
 
 /* brlapi_waitForAck */
 /* Wait for an acknowledgement, must be called with brlapi_fd_mutex locked */
-static int brlapi_waitForAck()
+static int brlapi_waitForAck(void)
 {
   return brlapi_waitForPacket(BRLPACKET_ACK, NULL, 0);
 }
@@ -195,6 +225,196 @@ static int brlapi_writePacketWaitForAck(int fd, brl_type_t type, const void *buf
   return res;
 }
 
+/* Function: tryHostName */
+/* Tries to connect to the given hostname. */
+static int tryHostName(char *hostName) {
+  int addrfamily;
+  char *hostname = NULL;
+  char *port;
+
+#ifdef WINDOWS
+  if (WSAStartup(
+#ifdef HAVE_GETADDRINFO
+	  MAKEWORD(2,0),
+#else /* HAVE_GETADDRINFO */
+	  MAKEWORD(1,1),
+#endif /* HAVE_GETADDRINFO */
+	  &wsadata))
+    return -1;
+#endif /* WINDOWS */
+
+  addrfamily = brlapi_splitHost(hostName,&hostname,&port);
+
+#if defined(PF_LOCAL) && (!defined(WINDOWS) || defined(HAVE_FUNC_CREATENAMEDPIPE))
+  if (addrfamily == PF_LOCAL) {
+    int lpath = strlen(BRLAPI_SOCKETPATH),lport;
+    lport = strlen(port);
+#ifdef WINDOWS
+    {
+      char path[lpath+lport+1];
+      memcpy(path,BRLAPI_SOCKETPATH,lpath);
+      memcpy(path+lpath,port,lport+1);
+      while ((HANDLE) (fd = (int)CreateFile(path,GENERIC_READ|GENERIC_WRITE,
+	      FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL))
+	  == INVALID_HANDLE_VALUE) {
+	if (GetLastError() != ERROR_PIPE_BUSY) {
+	  brlapi_errfun="CreateFile";
+	  goto outlibc;
+	}
+	WaitNamedPipe(path,NMPWAIT_WAIT_FOREVER);
+      }
+    }
+#else /* WINDOWS */
+    {
+      struct sockaddr_un sa;
+      if (lpath+lport+1>sizeof(sa.sun_path)) {
+	brlapi_libcerrno=ENAMETOOLONG;
+	brlapi_errfun="path";
+	brlapi_errno = BRLERR_LIBCERR;
+	goto out;
+      }
+      if ((fd = socket(PF_LOCAL, SOCK_STREAM, 0))<0) {
+        brlapi_errfun="socket";
+        setSocketErrno();
+        goto outlibc;
+      }
+      sa.sun_family = AF_LOCAL;
+      memcpy(sa.sun_path,BRLAPI_SOCKETPATH,lpath);
+      memcpy(sa.sun_path+lpath,port,lport+1);
+      if (connect(fd, (struct sockaddr *) &sa, sizeof(sa))<0) {
+        brlapi_errfun="connect";
+        setSocketErrno();
+        goto outlibc;
+      }
+    }
+#endif /* WINDOWS */
+  } else {
+#else /* PF_LOCAL */
+  if (0) {} else {
+#endif /* PF_LOCAL */
+
+#ifdef HAVE_GETADDRINFO
+
+    struct addrinfo *res,*cur;
+    struct addrinfo hints;
+
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_family = PF_UNSPEC;
+    hints.ai_socktype = SOCK_STREAM;
+
+    brlapi_gaierrno = getaddrinfo(hostname, port, &hints, &res);
+    if (brlapi_gaierrno) {
+      brlapi_errno=BRLERR_GAIERR;
+      goto out;
+    }
+    for(cur = res; cur; cur = cur->ai_next) {
+      fd = socket(cur->ai_family, cur->ai_socktype, cur->ai_protocol);
+      if (fd<0) continue;
+      if (connect(fd, cur->ai_addr, cur->ai_addrlen)<0) {
+        close(fd);
+        fd = -1;
+        continue;
+      }
+      break;
+    }
+    freeaddrinfo(res);
+    if (!cur) {
+      brlapi_errno=BRLERR_CONNREFUSED;
+      goto out;
+    }
+
+#else /* HAVE_GETADDRINFO */
+
+    struct sockaddr_in addr;
+    struct hostent *he;
+
+    memset(&addr,0,sizeof(addr));
+    addr.sin_family = AF_INET;
+    if (!port)
+      addr.sin_port = htons(BRLAPI_SOCKETPORTNUM);
+    else {
+      char *c;
+      addr.sin_port = htons(strtol(port, &c, 0));
+      if (*c) {
+	struct servent *se;
+	
+	if (!(se = getservbyname(port,"tcp"))) {
+	  brlapi_gaierrno = h_errno;
+          brlapi_errno=BRLERR_GAIERR;
+	  goto out;
+	}
+	addr.sin_port = se->s_port;
+      }
+    }
+
+    if (!hostname) {
+      addr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+    } else if ((addr.sin_addr.s_addr = inet_addr(hostname)) == htonl(INADDR_NONE)) {
+      if (!(he = gethostbyname(hostname))) {
+	brlapi_gaierrno = h_errno;
+	brlapi_errno = BRLERR_GAIERR;
+	goto out;
+      }
+      if (he->h_addrtype != AF_INET) {
+#ifdef EAFNOSUPPORT
+        errno = EAFNOSUPPORT;
+#else /* EAFNOSUPPORT */
+        errno = EINVAL;
+#endif /* EAFNOSUPPORT */
+	brlapi_errfun = "gethostbyname";
+	goto outlibc;
+      }
+      if (he->h_length > sizeof(addr.sin_addr)) {
+        errno = EINVAL;
+	brlapi_errfun = "gethostbyname";
+	goto outlibc;
+      }
+      memcpy(&addr.sin_addr,he->h_addr,he->h_length);
+    }
+    
+    fd = socket(addr.sin_family, SOCK_STREAM, 0);
+    if (fd<0) {
+      brlapi_errfun = "socket";
+      setSocketErrno();
+      goto outlibc;
+    }
+    if (connect(fd, (struct sockaddr *) &addr, sizeof(addr))<0) {
+      brlapi_errfun = "connect";
+      setSocketErrno();
+      goto outlibc;
+    }
+
+#endif /* HAVE_GETADDRINFO */
+
+  }
+  free(hostname);
+  free(port);
+  return 0;
+
+outlibc:
+  brlapi_errno = BRLERR_LIBCERR;
+  brlapi_libcerrno = errno;
+  if (fd>=0) {
+    close(fd);
+    fd = -1;
+  }
+out:
+  free(hostname);
+  free(port);
+  return -1;
+}
+
+/* Function: tryAuthKey */
+/* Tries to authenticate to server with the given key. */
+static int tryAuthKey(authStruct *auth, size_t authKeyLength) {
+  int res;
+  res = brlapi_writePacket(fd, BRLPACKET_AUTHKEY, auth, sizeof(auth->protocolVersion)+authKeyLength);
+  memset(&auth->key, 0, authKeyLength); /* Forget authorization key ASAP */
+  if (res<0) return -1;
+  if ((brlapi_waitForAck())<0) return -1;
+  return 0;
+}
+
 /* Function : updateSettings */
 /* Updates the content of a brlapi_settings_t structure according to */
 /* another structure of the same type */
@@ -213,11 +433,7 @@ int brlapi_initializeConnection(const brlapi_settings_t *clientSettings, brlapi_
 {
   unsigned char packet[BRLAPI_MAXPACKETSIZE];
   authStruct *auth = (authStruct *) packet;
-  int addrfamily;
-  char *hostname = NULL;
-  char *port;
   size_t authKeyLength;
-  int err;
 
   brlapi_settings_t settings = { BRLAPI_DEFAUTHPATH, ":0" };
   brlapi_settings_t envsettings = { getenv("BRLAPI_AUTHPATH"), getenv("BRLAPI_HOSTNAME") };
@@ -227,83 +443,45 @@ int brlapi_initializeConnection(const brlapi_settings_t *clientSettings, brlapi_
   updateSettings(&settings, clientSettings);
   if (usedSettings!=NULL) updateSettings(usedSettings, &settings);
 
-  if ((err=brlapi_loadAuthKey(settings.authKey,&authKeyLength,(void *) &auth->key))<0)
-    return err;
-
-  auth->protocolVersion = BRLAPI_PROTOCOL_VERSION;
-
-  addrfamily=brlapi_splitHost(settings.hostName,&hostname,&port);
-
   pthread_mutex_lock(&brlapi_fd_mutex);
 
-  if (addrfamily == PF_LOCAL) {
-    struct sockaddr_un sa;
-    int lpath = strlen(BRLAPI_SOCKETPATH),lport;
-    lport = strlen(port);
-    if (lpath+lport+1>sizeof(sa.sun_path)) {
-      brlapi_libcerrno=ENAMETOOLONG;
-      brlapi_libcerrfun="brlapi_initializeConnection";
-      brlapi_errno = BRLERR_LIBCERR;
-      return -1;
+retry:
+  if (tryHostName(settings.hostName)<0) {
+    brlapi_error_t error = brlapi_error;
+    if (tryHostName(settings.hostName="127.0.0.1:0")<0
+#ifdef AF_INET6
+      && tryHostName(settings.hostName="::1:0")<0
+#endif /* AF_INET6 */
+      ) {
+      brlapi_error = error;
+      goto out;
     }
-    if ((fd = socket(PF_LOCAL, SOCK_STREAM, 0))<0) {
-      brlapi_errno = BRLERR_LIBCERR;
-      brlapi_libcerrno = errno;
-      return fd;
+    if (usedSettings) usedSettings->hostName = settings.hostName;
+  }
+
+  auth->protocolVersion = htonl(BRLAPI_PROTOCOL_VERSION);
+
+  /* try with an empty key as well, in case no authentication is needed */
+  if (!settings.authKey) {
+    if (tryAuthKey(auth,0)<0)
+      goto outfd;
+    if (usedSettings) usedSettings->authKey = "none";
+  } else if (brlapi_loadAuthKey(settings.authKey,&authKeyLength,(void *) &auth->key)<0) {
+    brlapi_error_t error = brlapi_error;
+    if (tryAuthKey(auth,0)<0) {
+      brlapi_error = error;
+      goto outfd;
     }
-    sa.sun_family = AF_UNIX;
-    memcpy(sa.sun_path,BRLAPI_SOCKETPATH,lpath);
-    memcpy(sa.sun_path+lpath,port,lport+1);
-    if (connect(fd, (struct sockaddr *) &sa, sizeof(sa))<0) {
-      brlapi_errno = BRLERR_LIBCERR;
-      brlapi_libcerrno = errno;
+    if (usedSettings) usedSettings->authKey = "none";
+  } else if (tryAuthKey(auth,authKeyLength)<0) {
+    if (brlapi_errno != BRLERR_CONNREFUSED)
+      goto outfd;
+    if (fd>=0) {
       close(fd);
       fd = -1;
-      return -1;
     }
-  } else {
-    struct addrinfo *res,*cur;
-    struct addrinfo hints;
-    memset(&hints, 0, sizeof(hints));
-    hints.ai_family = PF_UNSPEC;
-    hints.ai_socktype = SOCK_STREAM;
-
-    gai_error = getaddrinfo(hostname, port, &hints, &res);
-    free(hostname);
-    free(port);
-    if (gai_error) {
-      brlapi_errno=BRLERR_GAIERR;
-      return -1;
-    }
-    for(cur = res; cur; cur = cur->ai_next) {
-      fd = socket(cur->ai_family, cur->ai_socktype, cur->ai_protocol);
-      if (fd<0) continue;
-      if (connect(fd, cur->ai_addr, cur->ai_addrlen)<0) {
-        close(fd);
-        fd = -1;
-        continue;
-      }
-      break;
-    }
-    freeaddrinfo(res);
-    if (!cur) {
-      pthread_mutex_unlock(&brlapi_fd_mutex);
-      brlapi_errno=BRLERR_CONNREFUSED;
-      return -1;
-    }
-  }
-
-  if ((err=brlapi_writePacket(fd, BRLPACKET_AUTHKEY, packet, sizeof(auth->protocolVersion)+authKeyLength))<0) {
-    pthread_mutex_unlock(&brlapi_fd_mutex);
-    close(fd);
-    fd = -1;
-    return err;
-  }
-  if ((err=brlapi_waitForAck())<0) {
-    pthread_mutex_unlock(&brlapi_fd_mutex);
-    close(fd);
-    fd = -1;
-    return err;
+    settings.authKey = NULL;
+    goto retry;
   }
   pthread_mutex_unlock(&brlapi_fd_mutex);
 
@@ -311,11 +489,20 @@ int brlapi_initializeConnection(const brlapi_settings_t *clientSettings, brlapi_
   state = STCONNECTED;
   pthread_mutex_unlock(&stateMutex);
   return fd;
+
+outfd:
+  if (fd>=0) {
+    close(fd);
+    fd = -1;
+  }
+out:
+  pthread_mutex_unlock(&brlapi_fd_mutex);
+  return -1;
 }
 
 /* brlapi_closeConnection */
 /* Cleanly close the socket */
-void brlapi_closeConnection()
+void brlapi_closeConnection(void)
 {
   pthread_mutex_lock(&stateMutex);
   state = 0;
@@ -324,15 +511,27 @@ void brlapi_closeConnection()
   close(fd);
   fd = -1;
   pthread_mutex_unlock(&brlapi_fd_mutex);
+#ifdef WINDOWS
+  WSACleanup();
+#endif /* WINDOWS */
 }
 
 /* brlapi_getRaw */
 /* Switch to Raw mode */
-int brlapi_getRaw()
+int brlapi_getRaw(const char *driver)
 {
   int res;
-  const uint32_t magic = htonl(BRLRAW_MAGIC);
-  res = brlapi_writePacketWaitForAck(fd, BRLPACKET_GETRAW, &magic, sizeof(magic));
+  unsigned char packet[BRLAPI_MAXPACKETSIZE];
+  getRawPacket_t *rawPacket = (getRawPacket_t *) packet;
+  unsigned int n = strlen(driver);
+  if (n>BRLAPI_MAXNAMELENGTH) {
+    brlapi_errno = BRLERR_INVALID_PARAMETER;
+    return -1;
+  }
+  rawPacket->magic = htonl(BRLRAW_MAGIC);
+  rawPacket->nameLength = n;
+  memcpy(&rawPacket->name, driver, n);
+  res = brlapi_writePacketWaitForAck(fd, BRLPACKET_GETRAW, packet, sizeof(uint32_t)+1+n);
   if (res!=-1) {
     pthread_mutex_lock(&stateMutex);
     state |= STRAW;
@@ -343,7 +542,7 @@ int brlapi_getRaw()
 
 /* brlapi_leaveRaw */
 /* Leave Raw mode */
-int brlapi_leaveRaw()
+int brlapi_leaveRaw(void)
 {
   int res = brlapi_writePacketWaitForAck(fd, BRLPACKET_LEAVERAW, NULL, 0);
   pthread_mutex_lock(&stateMutex);
@@ -354,7 +553,7 @@ int brlapi_leaveRaw()
 
 /* brlapi_sendRaw */
 /* Send a Raw Packet */
-ssize_t brlapi_sendRaw(const unsigned char *buf, size_t size)
+ssize_t brlapi_sendRaw(const void *buf, size_t size)
 {
   ssize_t res;
   pthread_mutex_lock(&brlapi_fd_mutex);
@@ -365,7 +564,7 @@ ssize_t brlapi_sendRaw(const unsigned char *buf, size_t size)
 
 /* brlapi_recvRaw */
 /* Get a Raw packet */
-ssize_t brlapi_recvRaw(unsigned char *buf, size_t size)
+ssize_t brlapi_recvRaw(void *buf, size_t size)
 {
   ssize_t res;
   pthread_mutex_lock(&brlapi_fd_mutex);
@@ -393,22 +592,20 @@ static ssize_t brlapi_request(brl_type_t request, void *packet, size_t size)
 
 /* Function : brlapi_getDriverId */
 /* Identify the driver used by brltty */
-int brlapi_getDriverId(unsigned char *id, size_t n)
+int brlapi_getDriverId(char *id, size_t n)
 {
-  unsigned char packet[BRLAPI_MAXPACKETSIZE];
-  ssize_t res = brlapi_request(BRLPACKET_GETDRIVERID, packet, sizeof(packet));
-  if (res<0) return -1;
-  return snprintf(id, n, "%s", packet);
+  ssize_t res = brlapi_request(BRLPACKET_GETDRIVERID, id, n);
+  if ((res>0) && (res<=n)) id[res-1] = '\0';
+  return res; 
 }
 
 /* Function : brlapi_getDriverName */
 /* Name of the driver used by brltty */
-int brlapi_getDriverName(unsigned char *name, size_t n)
+int brlapi_getDriverName(char *name, size_t n)
 {
-  unsigned char packet[BRLAPI_MAXPACKETSIZE];
-  ssize_t res = brlapi_request(BRLPACKET_GETDRIVERNAME, packet, sizeof(packet));
-  if (res<0) return -1;
-  return snprintf(name, n, "%s", packet);
+  ssize_t res = brlapi_request(BRLPACKET_GETDRIVERNAME, name, n);
+  if ((res>0) && (res<=n)) name[res-1] = '\0';
+  return res;
 }
 
 /* Function : brlapi_getDisplaySize */
@@ -419,7 +616,7 @@ int brlapi_getDisplaySize(unsigned int *x, unsigned int *y)
   ssize_t res;
 
   if (brlx*brly) { *x = brlx; *y = brly; return 0; }
-  res = brlapi_request(BRLPACKET_GETDISPLAYSIZE, displaySize, sizeof(displaySize)); 
+  res = brlapi_request(BRLPACKET_GETDISPLAYSIZE, displaySize, sizeof(displaySize));
   if (res==-1) { return -1; }
   brlx = ntohl(displaySize[0]);
   brly = ntohl(displaySize[1]);
@@ -427,71 +624,83 @@ int brlapi_getDisplaySize(unsigned int *x, unsigned int *y)
   return 0;
 }
 
-/* Function : brlapi_getControllingTty */
+/* Function : getControllingTty */
 /* Returns the number of the caller's controlling terminal */
 /* -1 if error or unknown */
-static int brlapi_getControllingTty()
+static int getControllingTty(void)
 {
   int tty;
   const char *env;
 
-#ifdef linux
-  {
-    int vt = 0;
-    pid_t pid = getpid();
-    while (pid != 1) {
-      int ok = 0;
-      char path[0X40];
-      FILE *stream;
-
-      int process;
-      char command[0X200];
-      char status;
-      int parent;
-      int group;
-      int session;
-      int tty;
-
-      snprintf(path, sizeof(path), "/proc/%d/stat", pid);
-      if ((stream = fopen(path, "r"))) {
-        if (fscanf(stream, "%d %s %c %d %d %d %d", &process, command, &status, &parent, &group, &session, &tty) >= 7)
-          if (process == pid)
-            ok = 1;
-        fclose(stream);
-      }
-      if (!ok) break;
-
-      if (major(tty) == TTY_MAJOR) {
-        vt = minor(tty);
-        break;
-      }
-
-      pid = parent;
-    }
-    if ((vt >= 1) && (vt <= MAXIMUM_VIRTUAL_CONSOLE)) return vt;
-  }
-#endif /* linux */
   /*if ((env = getenv("WINDOW")) && sscanf(env, "%u", &tty) == 1) return tty;*/
   if ((env = getenv("WINDOWID")) && sscanf(env, "%u", &tty) == 1) return tty;
   if ((env = getenv("CONTROLVT")) && sscanf(env, "%u", &tty) == 1) return tty;
+
+#ifdef WINDOWS
+#ifdef HAVE_FUNC_GETCONSOLEWINDOW
+  /* really good guess */
+  if ((tty = (int) GetConsoleWindow())) return tty;
+#endif /* HAVE_FUNC_GETCONSOLEWINDOW */
+  if ((tty = (int) GetActiveWindow()) || (tty = (int) GetFocus())) {
+    /* good guess, but need to get back up to parent window */
+    HWND root = GetDesktopWindow();
+    HWND tmp = (HWND) tty;
+    while (1) {
+      tmp = GetParent(tmp);
+      if (!tmp || tmp == root) return tty;
+      tty = (int) tmp;
+    }
+  }
+  /* poor guess: assumes that focus is here */
+  if ((tty = (int) GetForegroundWindow())) return tty;
+#endif /* WINDOWS */
+
+#ifdef linux
+  {
+    FILE *stream;
+
+    if ((stream = fopen("/proc/self/stat", "r"))) {
+      int vt;
+      int ok = fscanf(stream, "%*d %*s %*c %*d %*d %*d %d", &tty) == 1;
+      fclose(stream);
+
+      if (ok && (major(tty) == TTY_MAJOR)) {
+        vt = minor(tty);
+        if ((vt >= 1) && (vt <= MAXIMUM_VIRTUAL_CONSOLE)) return vt;
+      }
+    }
+  }
+#endif /* linux */
+
   return -1;
 }
 
 /* Function : brlapi_getTty */
 /* Takes control of a tty */
-int brlapi_getTty(int tty, int how)
+int brlapi_getTty(int tty, const char *how)
+{
+  /* Determine which tty to take control of */
+  if (tty<=0) tty = getControllingTty();
+  /* 0 can be a valid screen WINDOW
+  0xffffffff can not be a valid WINDOWID (top 3 bits guaranteed to be zero) */
+  if (tty<0) { brlapi_errno=BRLERR_UNKNOWNTTY; return -1; }
+  
+  if (brlapi_getTtyPath(&tty, 1, how)) return -1;
+
+  return tty;
+}
+
+/* Function : brlapi_getTtyPath */
+/* Takes control of a tty path */
+int brlapi_getTtyPath(int *ttys, int nttys, const char *how)
 {
-  uint32_t uints[BRLAPI_MAXPACKETSIZE/sizeof(uint32_t)],*curuints=uints;
   int res;
+  unsigned char packet[BRLAPI_MAXPACKETSIZE], *p;
+  uint32_t *nbTtys = (uint32_t *) packet, *t = nbTtys+1;
   char *ttytreepath,*ttytreepathstop;
   int ttypath;
+  unsigned int n;
 
-  /* Determine which tty to take control of */
-  if (tty<=0) truetty = brlapi_getControllingTty(); else truetty = tty;
-  // 0 can be a valid screen WINDOW
-  // 0xffffffff can not be a valid WINDOWID (top 3 bits guaranteed to be zero)
-  if (truetty<0) { brlapi_errno=BRLERR_UNKNOWNTTY; return -1; }
-  
   if (brlapi_getDisplaySize(&brlx, &brly)<0) return -1;
   
   /* Clear key buffer before taking the tty, just in case... */
@@ -500,30 +709,43 @@ int brlapi_getTty(int tty, int how)
   pthread_mutex_unlock(&keybuf_mutex);
 
   /* OK, Now we know where we are, so get the effective control of the terminal! */
+  *nbTtys = 0;
   ttytreepath = getenv("WINDOWSPATH");
   if (ttytreepath)
-  for(; *ttytreepath && curuints-uints+2<=BRLAPI_MAXPACKETSIZE/sizeof(uint32_t);
-      *curuints++ = htonl(ttypath), ttytreepath = ttytreepathstop+1) {
+  for(; *ttytreepath && t-(nbTtys+1)<=BRLAPI_MAXPACKETSIZE/sizeof(uint32_t);
+    *t++ = htonl(ttypath), (*nbTtys)++, ttytreepath = ttytreepathstop+1) {
     ttypath=strtol(ttytreepath,&ttytreepathstop,0);
     /* TODO: log it out. check overflow/underflow & co */
     if (ttytreepathstop==ttytreepath) break;
   }
 
-  *curuints++ = htonl(truetty); 
-  *curuints++ = htonl(how);
-  if ((res=brlapi_writePacketWaitForAck(fd,BRLPACKET_GETTTY,&uints[0],(curuints-uints)*sizeof(uint32_t)))<0)
+  for (n=0; n<nttys; n++) *t++ = htonl(ttys[n]);
+  (*nbTtys)+=nttys;
+
+  *nbTtys = htonl(*nbTtys);
+  p = (unsigned char *) t;
+  if (how==NULL) n = 0; else n = strlen(how);
+  if (n>BRLAPI_MAXNAMELENGTH) {
+    brlapi_errno = BRLERR_INVALID_PARAMETER;
+    return -1;
+  }
+  *p = n;
+  p++;
+  memcpy(p, how, n);
+  p += n;
+  if ((res=brlapi_writePacketWaitForAck(fd,BRLPACKET_GETTTY,packet,(p-packet)))<0)
     return res;
 
   pthread_mutex_lock(&stateMutex);
   state |= STCONTROLLINGTTY;
   pthread_mutex_unlock(&stateMutex);
 
-  return truetty;
+  return 0;
 }
 
 /* Function : brlapi_leaveTty */
 /* Gives back control of our tty to brltty */
-int brlapi_leaveTty()
+int brlapi_leaveTty(void)
 {
   int res;
   brlx = 0; brly = 0;
@@ -549,39 +771,95 @@ int brlapi_setFocus(int tty)
 
 /* Function : brlapi_writeText */
 /* Writes a string to the braille display */
-int brlapi_writeText(int cursor, const unsigned char *str)
+int brlapi_writeText(int cursor, const char *str)
 {
   int dispSize = brlx * brly;
-  unsigned int min, i;
+  unsigned int min;
   unsigned char packet[BRLAPI_MAXPACKETSIZE];
   writeStruct *ws = (writeStruct *) packet;
   unsigned char *p = &ws->data;
+  char *locale;
   int res;
   if ((dispSize == 0) || (dispSize > BRLAPI_MAXPACKETSIZE/4)) {
     brlapi_errno=BRLERR_INVALID_PARAMETER;
     return -1;
   }
+  locale = setlocale(LC_CTYPE,NULL);
   ws->flags = 0;
-  if (str==NULL) {
-    if (cursor==-1) goto send;
-  } else {
+  if (str) {
+    size_t len;
+    uint32_t *size;
     ws->flags |= BRLAPI_WF_TEXT;
-    min = MIN( strlen(str), dispSize);
-    strncpy(p,str,min);
-    p += min;
-    for (i = min; i<dispSize; i++,p++) *p = ' ';
+    size = (uint32_t *) p;
+    p += sizeof(*size);
+    len = strlen(str);
+#if !defined(WINDOWS) || defined(HAVE_LIBMSVCP60)
+    if (locale && strcmp(locale,"C")) {
+      mbstate_t ps;
+      size_t eaten;
+      unsigned i;
+      memset(&ps,0,sizeof(ps));
+      for (min=0;min<dispSize;min++) {
+	eaten = mbrlen(str,len,&ps);
+	switch(eaten) {
+	  case (size_t)(-2):
+	    errno = EILSEQ;
+	  case (size_t)(-1):
+	    brlapi_libcerrno = errno;
+	    brlapi_errfun = "mbrlen";
+	    brlapi_errno = BRLERR_LIBCERR;
+	    return -1;
+	  case 0:
+	    goto endcount;
+	}
+	memcpy(p, str, eaten);
+	p += eaten;
+	str += eaten;
+	len -= eaten;
+      }
+endcount:
+      for (i = min; i<dispSize; i++) p += wcrtomb((char *)p, L' ', &ps);
+    } else
+#endif /* HAVE_LIBMSVCP60 */
+    {
+      min = MIN(len, dispSize);
+      memcpy(p, str, min);
+      p += min;
+      memset(p, ' ', dispSize-min);
+      p += dispSize-min;
+    }
+    *size = htonl((p-(unsigned char *)(size+1)));
   }
   if ((cursor>=0) && (cursor<=dispSize)) {
     ws->flags |= BRLAPI_WF_CURSOR;
     *((uint32_t *) p) = htonl(cursor);
-    p += sizeof(cursor);
-  } else {
+    p += sizeof(uint32_t);
+  } else if (cursor!=-1) {
     brlapi_errno = BRLERR_INVALID_PARAMETER;
     return -1;
   }
-  send:
+
+  if (locale && strcmp(locale,"C")) {
+    /* not default locale, tell charset to server */
+#ifdef WINDOWS
+    UINT CP;
+    if ((CP = GetACP() || (CP = GetOEMCP()))) {
+      *p++ = 'C';
+      *p++ = 'P';
+      p += sprintf(p, "%d", CP);
+    }
+#else /* WINDOWS */
+    char *lang = nl_langinfo(CODESET);
+    size_t len = strlen(lang);
+    ws->flags |= BRLAPI_WF_CHARSET;
+    *p++ = len;
+    memcpy(p, lang, len);
+    p += len;
+#endif /* WINDOWS */
+  }
+  ws->flags = htonl(ws->flags);
   pthread_mutex_lock(&brlapi_fd_mutex);
-  res=brlapi_writePacket(fd,BRLPACKET_WRITE,packet,sizeof(ws->flags)+(p-&ws->data));
+  res = brlapi_writePacket(fd,BRLPACKET_WRITE,packet,sizeof(ws->flags)+(p-&ws->data));
   pthread_mutex_unlock(&brlapi_fd_mutex);
   return res;
 }
@@ -598,7 +876,7 @@ int brlapi_writeDots(const unsigned char *dots)
     return -1;
   }
   ws.displayNumber = -1;
-  ws.regionBegin = 0; ws.regionEnd = 0;
+  ws.regionBegin = 0; ws.regionSize = 0;
   ws.text = malloc(size);
   if (ws.text==NULL) {
     brlapi_errno = BRLERR_NOMEM;
@@ -614,6 +892,7 @@ int brlapi_writeDots(const unsigned char *dots)
   memcpy(ws.attrOr, dots, size);
   ws.attrAnd = NULL;
   ws.cursor = 0;
+  ws.charset = NULL;
   res = brlapi_write(&ws);
   free(ws.text);
   free(ws.attrOr);
@@ -625,44 +904,57 @@ int brlapi_writeDots(const unsigned char *dots)
 int brlapi_write(const brlapi_writeStruct *s)
 {
   int dispSize = brlx * brly;
-  unsigned int rbeg, rend, strLen;
+  unsigned int rbeg, rsiz, strLen;
   unsigned char packet[BRLAPI_MAXPACKETSIZE];
   writeStruct *ws = (writeStruct *) packet;
   unsigned char *p = &ws->data;
   int res;
   ws->flags = 0;
   if (s==NULL) goto send;
-  if ((1<=s->regionBegin) && (s->regionBegin<=dispSize) && (1<=s->regionEnd) && (s->regionEnd<=dispSize)) {
-    if (s->regionBegin>s->regionEnd) return 0;
-    rbeg = s->regionBegin; rend = s->regionEnd;
+  rbeg = s->regionBegin;
+  rsiz = s->regionSize;
+  if ((1<=rbeg) && (rbeg<=dispSize) && (1<=rbeg+rsiz-1) && (rbeg+rsiz-1<=dispSize)) {
+    if (rsiz == 0) return 0;
     ws->flags |= BRLAPI_WF_REGION;
     *((uint32_t *) p) = htonl(rbeg); p += sizeof(uint32_t);
-    *((uint32_t *) p) = htonl(rend); p += sizeof(uint32_t);
+    *((uint32_t *) p) = htonl(rsiz); p += sizeof(uint32_t);
   } else {
-    rbeg = 1; rend = dispSize;
+    rbeg = 1; rsiz = dispSize;
   }
-  strLen = (rend-rbeg) + 1;
   if (s->text) {
+    strLen = rsiz;
+    *((uint32_t *) p) = htonl(strLen); p += sizeof(uint32_t);
     ws->flags |= BRLAPI_WF_TEXT;
     memcpy(p, s->text, strLen);
     p += strLen;
   }
   if (s->attrAnd) {
     ws->flags |= BRLAPI_WF_ATTR_AND;
-    memcpy(p, s->attrAnd, strLen);
-    p += strLen;
+    memcpy(p, s->attrAnd, rsiz);
+    p += rsiz;
   }
   if (s->attrOr) {
     ws->flags |= BRLAPI_WF_ATTR_OR;
-    memcpy(p, s->attrOr, strLen);
-    p += strLen;
+    memcpy(p, s->attrOr, rsiz);
+    p += rsiz;
   }
   if ((s->cursor>=0) && (s->cursor<=dispSize)) {
     ws->flags |= BRLAPI_WF_CURSOR;
     *((uint32_t *) p) = htonl(s->cursor);
     p += sizeof(uint32_t);
+  } else if (s->cursor!=-1) {
+    brlapi_errno = BRLERR_INVALID_PARAMETER;
+    return -1;    
+  }
+  if (s->charset && *s->charset) {
+    strLen = strlen(s->charset);
+    *p++ = strLen;
+    ws->flags |= BRLAPI_WF_CHARSET;
+    strcpy((char *)p, s->charset);
+    p += strLen;
   }
   send:
+  ws->flags = htonl(ws->flags);
   pthread_mutex_lock(&brlapi_fd_mutex);
   res = brlapi_writePacket(fd,BRLPACKET_WRITE,packet,sizeof(ws->flags)+(p-&ws->data));
   pthread_mutex_unlock(&brlapi_fd_mutex);
@@ -711,6 +1003,8 @@ int brlapi_readKey(int block, brl_keycode_t *code)
   if (!block) {
     res = packetReady(fd);
     if (res<=0) {
+      if (res<0)
+	brlapi_errno = BRLERR_LIBCERR;
       pthread_mutex_unlock(&brlapi_fd_mutex);
       return res;
     }
@@ -749,11 +1043,12 @@ int brlapi_unignoreKeyRange(brl_keycode_t x, brl_keycode_t y)
 /* what = 0 for ignoring !0 for unignoring */
 static int ignore_unignore_key_set(int what, const brl_keycode_t *s, unsigned int n)
 {
-  size_t size = n*sizeof(brl_keycode_t);
-  if (size>BRLAPI_MAXPACKETSIZE) {
+  size_t size;
+  if (n>BRLAPI_MAXKEYSETSIZE) {
     brlapi_errno = BRLERR_INVALID_PARAMETER;
     return -1;
   }
+  size = n*sizeof(brl_keycode_t);
   return brlapi_writePacketWaitForAck(fd,(what ? BRLPACKET_UNIGNOREKEYSET : BRLPACKET_IGNOREKEYSET),s,size);
 }
 
@@ -781,113 +1076,150 @@ const char *brlapi_errlist[] = {
   "Illegal instruction",                /* BRLERR_ILLEGAL_INSTRUCTION */
   "Invalid parameter",                  /* BRLERR_INVALID_PARAMETER */
   "Invalid packet",                     /* BRLERR_INVALID_PACKET */
-  "Raw mode not supported by driver",   /* BRLERR_RAWNOTSUPP */
-  "Key codes not supported by driver",  /* BRLERR_KEYSNOTSUPP */
   "Connection refused",                 /* BRLERR_CONNREFUSED */
   "Operation not supported",            /* BRLERR_OPNOTSUPP */
   "getaddrinfo error",                  /* BRLERR_GAIERR */
   "libc error",                         /* BRLERR_LIBCERR */
-  "couldn't find out tty number",       /* BRLERR_UNKNOWNTTY */
-  "bad protocol version",               /* BRLERR_PROTOCOL_VERSION */
-  "unexpected end of file",             /* BRLERR_EOF */
-  "too many levels of recursion",       /* BRLERR_TOORECURSE */
-  "driver error",
+  "Couldn't find out tty number",       /* BRLERR_UNKNOWNTTY */
+  "Bad protocol version",               /* BRLERR_PROTOCOL_VERSION */
+  "Unexpected end of file",             /* BRLERR_EOF */
+  "Key file is empty",                  /* BRLERR_EMPTYKEY */
+  "Driver error",                       /* BRLERR_DRIVERERROR */
 };
 
 /* brlapi_nerr: last error number */
 const int brlapi_nerr = (sizeof(brlapi_errlist)/sizeof(char*)) - 1;
 
 /* brlapi_strerror: return error message */
-const char *brlapi_strerror(int err)
+const char *brlapi_strerror(const brlapi_error_t *error)
 {
-  if (err>=brlapi_nerr)
+  static char errmsg[128];
+  if (error->brlerrno>=brlapi_nerr)
     return "Unknown error";
-  else if (err==BRLERR_GAIERR)
-    return gai_strerror(gai_error);
-  else if (err==BRLERR_LIBCERR)
-    return strerror(brlapi_libcerrno);
-  else
-    return brlapi_errlist[err];
+  else if (error->brlerrno==BRLERR_GAIERR) {
+    snprintf(errmsg,sizeof(errmsg),"resolve: "
+#if defined(HAVE_GETADDRINFO) && defined(HAVE_GAI_STRERROR)
+	"%s\n", gai_strerror(error->exterrno.gaierrno)
+#elif defined(HAVE_HSTRERROR) && !defined(WINDOWS)
+	"%s\n", hstrerror(error->exterrno.gaierrno)
+#else
+	"%x\n", error->exterrno.gaierrno
+#endif
+	);
+    return errmsg;
+  }
+  else if (error->brlerrno==BRLERR_LIBCERR) {
+    snprintf(errmsg,sizeof(errmsg),"%s: %s", error->errfun, strerror(error->exterrno.libcerrno));
+    return errmsg;
+  } else
+    return brlapi_errlist[error->brlerrno];
 }
 
 /* brlapi_perror: error message printing */
 void brlapi_perror(const char *s)
 {
-  fprintf(stderr,"%s: %s\n",s,brlapi_strerror(brlapi_errno));
+  fprintf(stderr,"%s: %s\n",s,brlapi_strerror(&brlapi_error));
 }
 
 /* XXX functions mustn't use brlapi_errno after this since it #undefs it XXX */
 
-#ifdef brlapi_errno
-#undef brlapi_errno
-#endif
+#ifdef brlapi_error
+#undef brlapi_error
+#endif /* brlapi_error */
 
-int brlapi_errno;
-static int pthread_errno_ok;
+brlapi_error_t brlapi_error;
+static int pthread_error_ok;
 
 /* we need a per-thread errno variable, thanks to pthread_keys */
-static pthread_key_t errno_key;
+static pthread_key_t error_key;
 
 /* the key must be created at most once */
-static pthread_once_t errno_key_once = PTHREAD_ONCE_INIT;
+static pthread_once_t error_key_once = PTHREAD_ONCE_INIT;
 
-/* We need to declare these with __attribute__((weak)) to determine at runtime 
- * whether libpthread is used or not. We can't rely on the functions prototypes,
- * hence the use of typeof
+/* We need to declare these as weak external references to determine at runtime 
+ * whether libpthread is used or not. We also can't rely on the functions prototypes.
  */
+#if defined(WINDOWS)
+
+#elif defined(__GNUC__)
 #define WEAK_REDEFINE(name) extern typeof(name) name __attribute__((weak))
 WEAK_REDEFINE(pthread_key_create);
 WEAK_REDEFINE(pthread_once);
 WEAK_REDEFINE(pthread_getspecific);
 WEAK_REDEFINE(pthread_setspecific);
 
-static void errno_key_free(void *key)
+#elif defined(__sun__)
+#pragma weak pthread_key_create
+#pragma weak pthread_once
+#pragma weak pthread_getspecific
+#pragma weak pthread_setspecific
+
+#endif /* weak external references */
+
+static void error_key_free(void *key)
 {
   free(key);
 }
 
-static void errno_key_alloc(void)
+static void error_key_alloc(void)
 {
-  pthread_errno_ok=!pthread_key_create(&errno_key, errno_key_free);
+  pthread_error_ok=!pthread_key_create(&error_key, error_key_free);
 }
 
 /* how to get per-thread errno variable. This will be called by the macro
  * brlapi_errno */
-int *brlapi_errno_location(void)
+brlapi_error_t *brlapi_error_location(void)
 {
-  int *errnop;
+  brlapi_error_t *errorp;
+#ifndef WINDOWS
   if (pthread_once && pthread_key_create) {
-    pthread_once(&errno_key_once, errno_key_alloc);
-    if (pthread_errno_ok) {
-      if ((errnop=(int *) pthread_getspecific(errno_key)))
+#endif /* WINDOWS */
+    pthread_once(&error_key_once, error_key_alloc);
+    if (pthread_error_ok) {
+      if ((errorp=(brlapi_error_t *) pthread_getspecific(error_key)))
         /* normal case */
-        return errnop;
+        return errorp;
       else
         /* on the first time, must allocate it */
-        if ((errnop=malloc(sizeof(*errnop))) && !pthread_setspecific(errno_key,errnop))
-          return errnop;
-      }
+        if ((errorp=malloc(sizeof(*errorp))) && !pthread_setspecific(error_key,errorp))
+          return errorp;
     }
+#ifndef WINDOWS
+  }
+#endif /* WINDOWS */
   /* fall-back: shared errno :/ */
-  return &brlapi_errno;
+  return &brlapi_error;
 }
 
-brlapi_errorHandler_t brlapi_setErrorHandler(brlapi_errorHandler_t new)
+brlapi_exceptionHandler_t brlapi_setExceptionHandler(brlapi_exceptionHandler_t new)
 {
-  brlapi_errorHandler_t tmp;
-  pthread_mutex_lock(&brlapi_errorHandler_mutex);
-  tmp = brlapi_errorHandler;
-  if (new!=NULL) brlapi_errorHandler = new;
-  pthread_mutex_unlock(&brlapi_errorHandler_mutex);
+  brlapi_exceptionHandler_t tmp;
+  pthread_mutex_lock(&brlapi_exceptionHandler_mutex);
+  tmp = brlapi_exceptionHandler;
+  if (new!=NULL) brlapi_exceptionHandler = new;
+  pthread_mutex_unlock(&brlapi_exceptionHandler_mutex);
   return tmp;
 }
 
-void brlapi_defaultErrorHandler(int err, brl_type_t type, const void *packet, size_t size)
+int brlapi_strexception(char *buf, size_t n, int err, brl_type_t type, const void *packet, size_t size)
 {
-  const unsigned char *c;
-  fprintf(stderr, "Error: %s on %s request:\n",brlapi_strerror(err),brlapi_packetType(type));
-  if (size>16) size=16;
-  for (c=packet;c<(unsigned char *)packet+size;c++) fprintf(stderr,"%2x ", *c);
-  fprintf(stderr,"\n");
-  exit(1);
+  int chars = 16; /* Number of bytes to dump */
+  char hexString[3*chars+1];
+  int i, nbChars = MIN(chars, size);
+  char *p = hexString;
+  brlapi_error_t error = { .brlerrno = err };
+  for (i=0; i<nbChars; i++)
+    p += sprintf(p, "%02x ", ((char *) packet)[i]);
+  p--; /* Don't keep last space */
+  *p = '\0';
+  return snprintf(buf, n, "%s on %s request of size %d (%s)",
+    brlapi_strerror(&error), brlapi_packetType(type), size, hexString);
+}
+
+void brlapi_defaultExceptionHandler(int err, brl_type_t type, const void *packet, size_t size)
+{
+  char str[0X100];
+  brlapi_strexception(str,0X100, err, type, packet, size);
+  fprintf(stderr, "BrlAPI exception: %s\n", str);
+  abort();
 }
