@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,8 +15,8 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _AT2_H
-#define _AT2_H
+#ifndef BRLTTY_INCLUDED_AT2
+#define BRLTTY_INCLUDED_AT2
 
 #ifdef __cplusplus
 extern "C" {
@@ -29,4 +29,4 @@ extern int AT2_interpretCode (int *command, unsigned char byte);
 }
 #endif /* __cplusplus */
 
-#endif /* _AT2_H */
+#endif /* BRLTTY_INCLUDED_AT2 */
