@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,9 +15,7 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -31,18 +29,18 @@ static int beeperOpened = 0;
 static int openBeeper (int errorLevel) {
    if (!beeperOpened) {
       if (!canBeep()) {
-         LogPrint(LOG_DEBUG, "Cannot open beeper.");
+         LogPrint(LOG_DEBUG, "cannot open beeper.");
          return 0;
       }
       beeperOpened = 1;
-      LogPrint(LOG_DEBUG, "Beeper opened.");
+      LogPrint(LOG_DEBUG, "beeper opened.");
    }
    return 1;
 }
 
 static int playBeeper (int note, int duration) {
    if (beeperOpened) {
-      LogPrint(LOG_DEBUG, "Tone: msec=%d note=%d",
+      LogPrint(LOG_DEBUG, "tone: msec=%d note=%d",
                duration, note);
 
       if (!note) {
@@ -75,7 +73,7 @@ static void closeBeeper (void) {
    if (beeperOpened) {
       beeperOpened = 0;
       endBeep();
-      LogPrint(LOG_DEBUG, "Beeper closed.");
+      LogPrint(LOG_DEBUG, "beeper closed.");
    }
 }
 
