@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,8 +15,8 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _BRL_H
-#define _BRL_H
+#ifndef BRLTTY_INCLUDED_BRL
+#define BRLTTY_INCLUDED_BRL
 
 #ifdef __cplusplus
 extern "C" {
@@ -39,9 +39,10 @@ typedef enum {
 
 /* Braille information structure. */
 typedef struct {
-  int x, y;			/* the dimensions of the display */
-  int helpPage;			/* the page number within the help file */
+  unsigned int x, y;			/* the dimensions of the display */
+  unsigned int helpPage;			/* the page number within the help file */
   unsigned char *buffer;	/* the contents of the display */
+  int cursor;		/* the position of the cursor within the display */
   unsigned isCoreBuffer:1;	/* the core allocated the buffer */
   unsigned resizeRequired:1;	/* the display size has changed */
   unsigned int writeDelay;
@@ -62,7 +63,6 @@ extern void clearStatusCells (BrailleDisplay *brl);
 extern void setStatusText (BrailleDisplay *brl, const char *text);
 
 extern int readBrailleCommand (BrailleDisplay *, BRL_DriverCommandContext);
-#define IS_DELAYED_COMMAND(cmd) (((cmd) & BRL_FLG_REPEAT_DELAY) && !((cmd) & BRL_FLG_REPEAT_INITIAL))
 
 typedef enum {
   BF_MINIMUM,
@@ -77,24 +77,25 @@ typedef enum {
  * with pointers to all the functions and variables.
  */
 typedef struct {
-  const char *name;			/* name of driver */
-  const char *identifier;		/* name of driver */
-  const char *date;		/* compilation date of driver */
-  const char *time;		/* compilation time of driver */
-  const char *const *parameters;	/* user-supplied driver parameters */
-  const char *helpFile;		/* name of help file */
-  int statusStyle;		/* prefered status cells mode */
+  const char *name;
+  const char *code;
+  const char *comment;
+  const char *date;
+  const char *time;
+  const char *const *parameters;
+  const char *helpFile;
+  int statusStyle;
 
   /* Routines provided by the braille driver library: */
-  void (*identify) (void);	/* print start-up messages */
-  int (*open) (BrailleDisplay *, char **parameters, const char *);	/* initialise Braille display */
-  void (*close) (BrailleDisplay *);		/* close braille display */
-  int (*readCommand) (BrailleDisplay *, BRL_DriverCommandContext);		/* get key press from braille display */
-  void (*writeWindow) (BrailleDisplay *);		/* write to braille display */
-  void (*writeStatus) (BrailleDisplay *brl, const unsigned char *);	/* set status cells */
+  void (*identify) (void);
+  int (*open) (BrailleDisplay *, char **parameters, const char *);
+  void (*close) (BrailleDisplay *);
+  int (*readCommand) (BrailleDisplay *, BRL_DriverCommandContext);
+  void (*writeWindow) (BrailleDisplay *);
+  void (*writeStatus) (BrailleDisplay *brl, const unsigned char *);
 
   /* These require BRL_HAVE_VISUAL_DISPLAY. */
-  void (*writeVisual) (BrailleDisplay *);		/* write to braille display */
+  void (*writeVisual) (BrailleDisplay *);
 
   /* These require BRL_HAVE_PACKET_IO. */
   ssize_t (*readPacket) (BrailleDisplay *, unsigned char *, size_t);
@@ -106,25 +107,29 @@ typedef struct {
   int (*keyToCommand) (BrailleDisplay *, BRL_DriverCommandContext, int);
 
   /* These require BRL_HAVE_FIRMNESS. */
-  void (*firmness) (BrailleDisplay *brl, BrailleFirmness setting);		/* mute speech */
+  void (*firmness) (BrailleDisplay *brl, BrailleFirmness setting);
 } BrailleDriver;
 
-extern const BrailleDriver *loadBrailleDriver (const char *identifier, void **driverObject, const char *driverDirectory);
+extern const BrailleDriver *loadBrailleDriver (const char *code, void **driverObject, const char *driverDirectory);
 extern void identifyBrailleDrivers (void);
-extern int listBrailleDrivers (const char *directory);
 extern const BrailleDriver *braille;
 extern const BrailleDriver noBraille;
 
-typedef unsigned char TranslationTable[0X100];
+#define TRANSLATION_TABLE_SIZE 0X100
+typedef unsigned char TranslationTable[TRANSLATION_TABLE_SIZE];
 extern TranslationTable textTable;	 /* current text to braille translation table */
 extern TranslationTable untextTable;     /* current braille to text translation table */
 extern TranslationTable attributesTable; /* current attributes to braille translation table */
 extern void *contractionTable; /* current braille contraction table */
 
-typedef unsigned char DotsTable[8];
-extern void makeOutputTable (const DotsTable *dots, TranslationTable *table);
+extern void makeUntextTable (void);
+
+#define DOTS_TABLE_SIZE 8
+typedef unsigned char DotsTable[DOTS_TABLE_SIZE];
+extern void makeOutputTable (const DotsTable dots, TranslationTable table);
 
 /* Formatting of status cells. */
+extern unsigned char lowerDigit (unsigned char upper);
 extern const unsigned char landscapeDigits[11];
 extern int landscapeNumber (int x);
 extern int landscapeFlag (int number, int on);
@@ -144,4 +149,4 @@ extern void setBrailleFirmness (BrailleDisplay *brl, int setting);
 }
 #endif /* __cplusplus */
 
-#endif /* _BRL_H */
+#endif /* BRLTTY_INCLUDED_BRL */
