@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,17 +15,16 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _BRLTTY_H
-#define _BRLTTY_H
+#ifndef BRLTTY_INCLUDED_BRLTTY
+#define BRLTTY_INCLUDED_BRLTTY
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
+#include "cmd.h"
 #include "brl.h"
 
 typedef enum {
@@ -111,6 +110,7 @@ extern short vwinshift;			/* Window vertical distance */
 extern int updateInterval;
 extern int messageDelay;
 
+extern void testProgramTermination (void);
 extern void startup (int argc, char *argv[]);
 extern int loadPreferences (int change);
 extern int savePreferences (void);
@@ -119,18 +119,20 @@ extern void updatePreferences (void);
 extern void restartBrailleDriver (void);
 extern void restartSpeechDriver (void);
 
-extern int getCommand (BRL_DriverCommandContext context);
 extern int readCommand (BRL_DriverCommandContext context);
+extern void resetAutorepeat (void);
+extern void handleAutorepeat (int *command, RepeatState *state);
 
 extern void api_identify (void);
 extern int api_open (BrailleDisplay *brl, char **parameters);
 extern void api_close (BrailleDisplay *brl);
 extern void api_link (void);
 extern void api_unlink (void);
+extern void api_flush (BrailleDisplay *brl, BRL_DriverCommandContext caller);
 extern const char *const api_parameters[];
 
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
 
-#endif /* _BRLTTY_H */
+#endif /* BRLTTY_INCLUDED_BRLTTY */
