@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,8 +15,10 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _CMD_H
-#define _CMD_H
+#ifndef BRLTTY_INCLUDED_CMD
+#define BRLTTY_INCLUDED_CMD
+
+#include <sys/time.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -32,8 +34,17 @@ extern const CommandEntry commandTable[];
 
 extern void describeCommand (int command, char *buffer, int size);
 
+typedef struct {
+  int command;
+  int timeout;
+  int started;
+  struct timeval time;
+} RepeatState;
+extern void resetRepeatState (RepeatState *state);
+extern void handleRepeatFlags (int *command, RepeatState *state, int delay, int interval);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
 
-#endif /* _CMD_H */
+#endif /* BRLTTY_INCLUDED_CMD */
