@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,8 +15,8 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _CTB_H
-#define _CTB_H
+#ifndef BRLTTY_INCLUDED_CTB
+#define BRLTTY_INCLUDED_CTB
 
 #ifdef __cplusplus
 extern "C" {
@@ -38,4 +38,4 @@ extern int contractText (
 }
 #endif /* __cplusplus */
 
-#endif /* _CTB_H */
+#endif /* BRLTTY_INCLUDED_CTB */
