@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,9 +15,7 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -30,7 +28,7 @@
  
 #include "misc.h"
 #include "ctb.h"
-#include "ctb_definitions.h"
+#include "ctb_internal.h"
 #include "brl.h"
 
 typedef struct {
@@ -79,6 +77,7 @@ static const char *const opcodeNames[CTO_None] = {
   "repeated",
 
   "largesign",
+  "lastlargesign",
   "word",
   "joinword",
   "lowword",
@@ -599,6 +598,7 @@ doOpcode:
 
     case CTO_Always:
     case CTO_LargeSign:
+    case CTO_LastLargeSign:
     case CTO_WholeWord:
     case CTO_JoinableWord:
     case CTO_LowWord:
