@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,9 +15,7 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -29,17 +27,20 @@
 #include "cut.h"
 
 /* Global state variables */
-unsigned char *cut_buffer = NULL;
-static short int beginColumn = 0, beginRow = 0;
-static unsigned int previousLength = 0;
+unsigned char *cutBuffer = NULL;
+size_t cutLength = 0;
+
+static int beginColumn = 0;
+static int beginRow = 0;
+static int beginOffset = -1;
 
 static unsigned char *
-cut (int fromColumn, int fromRow, int toColumn, int toRow) {
+cut (size_t *length, int fromColumn, int fromRow, int toColumn, int toRow) {
   unsigned char *newBuffer = NULL;
   int columns = toColumn - fromColumn + 1;
   int rows = toRow - fromRow + 1;
 
-  if ((columns >= 1) && (rows >= 1)) {
+  if ((columns >= 1) && (rows >= 1) && (beginOffset >= 0)) {
     unsigned char *fromBuffer = malloc(rows * columns);
     if (fromBuffer) {
       unsigned char *toBuffer = malloc(rows * (columns + 1));
@@ -48,124 +49,146 @@ cut (int fromColumn, int fromRow, int toColumn, int toRow) {
           unsigned char *fromAddress = fromBuffer;
           unsigned char *toAddress = toBuffer;
           int row;
+
           /* remove spaces at end of line, add return (except to last line),
-             and possibly remove non-printables... if any */
+           * and possibly remove non-printables... if any
+           */
           for (row=fromRow; row<=toRow; row++) {
             int spaces = 0;
             int column;
+
             for (column=fromColumn; column<=toColumn; column++, fromAddress++) {
-              if (iscntrl(*fromAddress) || isspace(*fromAddress))
+              if (iscntrl(*fromAddress) || isspace(*fromAddress)) {
                 spaces++;
-              else {
+              } else {
                 while (spaces) {
                   *(toAddress++) = ' ';
                   spaces--;
                 }
+
                 *(toAddress++) = *fromAddress;
               }
             }
+
             if (row != toRow) *(toAddress++) = '\r';
           }
-          *toAddress++ = 0;
 
           /* make a new permanent buffer of just the right size */
           {
-            int length = toAddress - toBuffer;
-            if ((newBuffer = (unsigned char *)malloc(length))) {
-              memcpy(newBuffer, toBuffer, length);
+            size_t newLength = toAddress - toBuffer;
+            if ((newBuffer = malloc(newLength))) {
+              memcpy(newBuffer, toBuffer, (*length = newLength));
             }
           }
         }
+        
         free(toBuffer);
       }
+
       free(fromBuffer);
     }
   }
+
   return newBuffer;
 }
 
 static int
-append (unsigned char *buffer) {
-  if (cut_buffer) {
-    unsigned char *newBuffer = malloc(previousLength + strlen(buffer) + 1);
+append (unsigned char *buffer, size_t length) {
+  if (cutBuffer) {
+    size_t newLength = beginOffset + length;
+    unsigned char *newBuffer = malloc(newLength);
     if (!newBuffer) return 0;
-    memcpy(newBuffer, cut_buffer, previousLength);
-    strcpy(newBuffer+previousLength, buffer);
+
+    memcpy(newBuffer, cutBuffer, beginOffset);
+    memcpy(newBuffer+beginOffset, buffer, length);
+
     free(buffer);
-    free(cut_buffer);
-    cut_buffer = newBuffer;
+    free(cutBuffer);
+
+    cutBuffer = newBuffer;
+    cutLength = newLength;
   } else {
-    cut_buffer = buffer;
+    cutBuffer = buffer;
+    cutLength = length;
   }
+
   playTune(&tune_cut_end);
   return 1;
 }
 
-static void
-start (int column, int row) {
+void
+cutBegin (int column, int row) {
+  if (cutBuffer) {
+    free(cutBuffer);
+    cutBuffer = NULL;
+  }
+  cutLength = 0;
+
+  cutAppend(column, row);
+}
+
+void
+cutAppend (int column, int row) {
   beginColumn = column;
   beginRow = row;
+  beginOffset = cutLength;
   playTune(&tune_cut_begin);
 }
 
-void
-cut_begin (int column, int row) {
-  if (cut_buffer) {
-    free(cut_buffer);
-    cut_buffer = NULL;
-  }
-  previousLength = 0;
-  start(column, row);
-}
-
-void
-cut_append (int column, int row) {
-  previousLength = cut_buffer? strlen(cut_buffer): 0;
-  start(column, row);
-}
-
 int
-cut_rectangle (int column, int row) {
-  unsigned char *buffer = cut(beginColumn, beginRow, column, row);
+cutRectangle (int column, int row) {
+  size_t length;
+  unsigned char *buffer = cut(&length, beginColumn, beginRow, column, row);
+
   if (buffer) {
-    if (append(buffer)) return 1;
+    if (append(buffer, length)) return 1;
     free(buffer);
   }
   return 0;
 }
 
 int
-cut_line (int column, int row) {
+cutLine (int column, int row) {
   ScreenDescription screen;
   describeScreen(&screen);
+
   {
-    int width = screen.cols;
-    int rightColumn = width - 1;
-    unsigned char *buffer = cut(0, beginRow, rightColumn, row);
+    int rightColumn = screen.cols - 1;
+    size_t length;
+    unsigned char *buffer = cut(&length, 0, beginRow, rightColumn, row);
+
     if (buffer) {
       if (column < rightColumn) {
-        int length = column + 1;
-        unsigned char *start = strrchr(buffer, '\r');
-        start = start? start+1: buffer;
-        if (length < strlen(start)) start[length] = 0;
+        unsigned char *start = buffer + length;
+        while (start != buffer) {
+          if (*--start == '\r') {
+            ++start;
+            break;
+          }
+        }
+
+        {
+          int adjustment = (column + 1) - (buffer + length - start);
+          if (adjustment < 0) length += adjustment;
+        }
       }
+
       if (beginColumn) {
-        unsigned char *start = strchr(buffer, '\r');
-        if (!start) start = buffer + strlen(buffer);
+        unsigned char *start = memchr(buffer, '\r', length);
+        if (!start) start = buffer + length;
         if ((start - buffer) > beginColumn) start = buffer + beginColumn;
-        if (start != buffer) memmove(buffer, start, strlen(start)+1);
+        if (start != buffer) memmove(buffer, start, (length -= start - buffer));
       }
-      if (append(buffer)) return 1;
+
+      if (append(buffer, length)) return 1;
       free(buffer);
     }
-    return 0;
   }
+
+  return 0;
 }
 
 int
-cut_paste (void) {
-  if (cut_buffer)
-    if (insertString(cut_buffer))
-      return 1;
-  return 0;
+cutPaste (void) {
+  return cutBuffer && insertCharacters((char *)cutBuffer, cutLength);
 }
