@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,22 +15,24 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _CUT_H
-#define _CUT_H
+#ifndef BRLTTY_INCLUDED_CUT
+#define BRLTTY_INCLUDED_CUT
 
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
-unsigned char *cut_buffer; /* for CMD_NXSEARCH */
-extern void cut_begin (int column, int row);
-extern void cut_append (int column, int row);
-extern int cut_rectangle (int column, int row);
-extern int cut_line (int column, int row);
-extern int cut_paste (void);
+extern unsigned char *cutBuffer;
+extern size_t cutLength;
+
+extern void cutBegin (int column, int row);
+extern void cutAppend (int column, int row);
+extern int cutRectangle (int column, int row);
+extern int cutLine (int column, int row);
+extern int cutPaste (void);
 
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
 
-#endif /* _CUT_H */
+#endif /* BRLTTY_INCLUDED_CUT */
