@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,9 +15,7 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -39,7 +37,7 @@ static int openFm (int errorLevel) {
 	 }
       }
       if (!ready) {
-         LogPrint(LOG_DEBUG, "Cannot open FM.");
+         LogPrint(LOG_DEBUG, "cannot open FM.");
          return 0;
       }
       LogPrint(LOG_DEBUG, "FM opened.");
@@ -49,7 +47,7 @@ static int openFm (int errorLevel) {
 }
 
 static int playFm (int note, int duration) {
-   LogPrint(LOG_DEBUG, "Tone: msec=%d note=%d",
+   LogPrint(LOG_DEBUG, "tone: msec=%d note=%d",
 	    duration, note);
    if (note)
       AL_playTone(channelNumber, (int)noteFrequencies[note], duration, prefs.fmVolume);
