@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,8 +15,8 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _MESSAGE_H
-#define _MESSAGE_H
+#ifndef BRLTTY_INCLUDED_MESSAGE
+#define BRLTTY_INCLUDED_MESSAGE
 
 #ifdef __cplusplus
 extern "C" {
@@ -38,4 +38,4 @@ extern void message (const char *, short);
 }
 #endif /* __cplusplus */
 
-#endif /* _MESSAGE_H */
+#endif /* BRLTTY_INCLUDED_MESSAGE */
