@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,8 +15,8 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _NOTES_H
-#define _NOTES_H
+#ifndef BRLTTY_INCLUDED_NOTES
+#define BRLTTY_INCLUDED_NOTES
 
 #ifdef __cplusplus
 extern "C" {
@@ -41,4 +41,4 @@ extern const unsigned int noteCount;
 }
 #endif /* __cplusplus */
 
-#endif /* _NOTES_H */
+#endif /* BRLTTY_INCLUDED_NOTES */
