@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,9 +15,7 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -56,7 +54,7 @@ static int openPcm (int errorLevel) {
       closePcmDevice(pcm);
       pcm = NULL;
    } else {
-      LogPrint(LOG_DEBUG, "Cannot open PCM.");
+      LogPrint(LOG_DEBUG, "cannot open PCM.");
    }
    return 0;
 }
@@ -149,7 +147,7 @@ static int playPcm (int note, int duration) {
          float stepSample = 4 * maxAmplitude / waveLength;
          float currSample = 0;
          if (waveLength <= 2) stepSample = 0;
-         LogPrint(LOG_DEBUG, "Tone: msec=%d smct=%lu note=%d",
+         LogPrint(LOG_DEBUG, "tone: msec=%d smct=%lu note=%d",
                   duration, sampleCount, note);
          while (sampleCount > 0) {
             do {
@@ -166,7 +164,7 @@ static int playPcm (int note, int duration) {
             } while ((stepSample < 0) || (currSample < 0) || (currSample > stepSample));
          }
       } else {
-         LogPrint(LOG_DEBUG, "Tone: msec=%d smct=%lu note=%d",
+         LogPrint(LOG_DEBUG, "tone: msec=%d smct=%lu note=%d",
                   duration, sampleCount, note);
          while (sampleCount > 0) {
             if (!writeAmplitude(0)) return 0;
