@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,8 +15,8 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _SCR_BASE_H
-#define _SCR_BASE_H
+#ifndef BRLTTY_INCLUDED_SCR_BASE
+#define BRLTTY_INCLUDED_SCR_BASE
 
 #ifdef __cplusplus
 extern "C" {
@@ -24,7 +24,7 @@ extern "C" {
 
 typedef struct {
   void (*describe) (ScreenDescription *);
-  unsigned char * (*read) (ScreenBox, unsigned char *, ScreenMode);
+  int (*read) (ScreenBox, unsigned char *, ScreenMode);
   int (*insert) (ScreenKey);
   int (*route) (int, int, int);
   int (*point) (int, int);
@@ -41,4 +41,4 @@ extern void initializeBaseScreen (BaseScreen *);
 }
 #endif /* __cplusplus */
 
-#endif /* _SCR_BASE_H */
+#endif /* BRLTTY_INCLUDED_SCR_BASE */
