@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,9 +15,7 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -72,7 +70,7 @@ describe_FrozenScreen (ScreenDescription *description) {
   *description = screenDescription;
 }
 
-static unsigned char *
+static int
 read_FrozenScreen (ScreenBox box, unsigned char *buffer, ScreenMode mode) {
   if (validateScreenBox(&box, screenDescription.cols, screenDescription.rows)) {
     unsigned char *screen = (mode == SCR_TEXT)? screenText: screenAttributes;
@@ -81,9 +79,14 @@ read_FrozenScreen (ScreenBox box, unsigned char *buffer, ScreenMode mode) {
       memcpy(buffer + (row * box.width),
              screen + ((box.top + row) * screenDescription.cols) + box.left,
              box.width);
-    return buffer;
+    return 1;
   }
-  return NULL;
+  return 0;
+}
+
+static int
+currentvt_FrozenScreen (void) {
+  return screenDescription.no;
 }
 
 void
@@ -91,6 +94,7 @@ initializeFrozenScreen (FrozenScreen *frozen) {
   initializeBaseScreen(&frozen->base);
   frozen->base.describe = describe_FrozenScreen;
   frozen->base.read = read_FrozenScreen;
+  frozen->base.currentvt = currentvt_FrozenScreen;
   frozen->open = open_FrozenScreen;
   frozen->close = close_FrozenScreen;
   screenText = NULL;
