@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,9 +15,7 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -220,6 +218,11 @@ getPageCount_HelpScreen (void) {
   return (fileDescriptor != -1)? fileHeader.pages: 0;
 }
 
+static int
+currentvt_HelpScreen (void) {
+  return userVirtualTerminal(1);
+}
+
 static void
 describe_HelpScreen (ScreenDescription *description) {
   const HelpPageEntry *page = &pageDescriptions[pageNumber];
@@ -227,10 +230,10 @@ describe_HelpScreen (ScreenDescription *description) {
   description->posy = cursorRow;
   description->cols = getBigEndian(page->width);
   description->rows = getBigEndian(page->height);
-  description->no = 0;	/* 0 is reserved for help screen */
+  description->no = currentvt_HelpScreen();
 }
 
-static unsigned char *
+static int
 read_HelpScreen (ScreenBox box, unsigned char *buffer, ScreenMode mode) {
   const HelpPageEntry *description = &pageDescriptions[pageNumber];
   if (validateScreenBox(&box, getBigEndian(description->width), getBigEndian(description->height))) {
@@ -244,9 +247,9 @@ read_HelpScreen (ScreenBox box, unsigned char *buffer, ScreenMode mode) {
     } else {
       memset(buffer, 0X07, box.width*box.height);
     }
-    return buffer;
+    return 1;
   }
-  return NULL;
+  return 0;
 }
 
 static int
@@ -313,6 +316,7 @@ route_HelpScreen (int column, int row, int screen) {
 void
 initializeHelpScreen (HelpScreen *help) {
   initializeBaseScreen(&help->base);
+  help->base.currentvt = currentvt_HelpScreen;
   help->base.describe = describe_HelpScreen;
   help->base.read = read_HelpScreen;
   help->base.insert = insert_HelpScreen;
