@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,9 +15,7 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -27,12 +25,6 @@
 #include "scr.h"
 #include "scr_main.h"
 
-static const char *const screenParameters[] = {NULL};
-static const char *const *
-parameters_MainScreen (void) {
-  return screenParameters;
-}
-
 static int
 prepare_MainScreen (char **parameters) {
   return 1;
@@ -52,12 +44,17 @@ static void
 close_MainScreen (void) {
 }
 
+static int
+uservt_MainScreen (int number) {
+  return 0 + number;
+}
+
 void
 initializeMainScreen (MainScreen *main) {
   initializeBaseScreen(&main->base);
-  main->parameters = parameters_MainScreen;
   main->prepare = prepare_MainScreen;
   main->open = open_MainScreen;
   main->setup = setup_MainScreen;
   main->close = close_MainScreen;
+  main->uservt = uservt_MainScreen;
 }
