@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,8 +15,8 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _SPK_DRIVER_H
-#define _SPK_DRIVER_H
+#ifndef BRLTTY_INCLUDED_SPK_DRIVER
+#define BRLTTY_INCLUDED_SPK_DRIVER
 
 #ifdef __cplusplus
 extern "C" {
@@ -24,7 +24,7 @@ extern "C" {
 
 /* this header file is used to create the driver structure
  * for a dynamically loadable speech driver.
- * SPKNAME, and SPKCODE must be defined - see driver make file
+ * SPKNAME, SPKCODE, and SPKCOMMENT must be defined - see driver make file
  */
 
 #include "spk.h"
@@ -51,11 +51,11 @@ static void spk_close (void);
 #endif /* SPK_HAVE_TRACK */
 
 #ifdef SPK_HAVE_RATE
-  static void spk_rate (int setting);		/* mute speech */
+  static void spk_rate (float setting);		/* mute speech */
 #endif /* SPK_HAVE_RATE */
 
 #ifdef SPK_HAVE_VOLUME
-  static void spk_volume (int setting);		/* mute speech */
+  static void spk_volume (float setting);		/* mute speech */
 #endif /* SPK_HAVE_VOLUME */
 
 #ifdef SPKPARMS
@@ -74,6 +74,7 @@ extern SPKCONST SpeechDriver SPKSYMBOL;
 SPKCONST SpeechDriver SPKSYMBOL = {
   STRINGIFY(SPKNAME),
   STRINGIFY(SPKCODE),
+  SPKCOMMENT,
   __DATE__,
   __TIME__,
 
@@ -116,4 +117,4 @@ SPKCONST SpeechDriver SPKSYMBOL = {
 }
 #endif /* __cplusplus */
 
-#endif /* _SPK_DRIVER_H */
+#endif /* BRLTTY_INCLUDED_SPK_DRIVER */
