@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,7 +15,7 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#warning beeps not available on this platform
+#warning console beep support not available on this platform
 
 int
 canBeep (void) {
