@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,9 +15,7 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -35,6 +33,35 @@
 #include "misc.h"
 #include "system.h"
 
+char *
+getProgramPath (void) {
+  char *path = NULL;
+  size_t size = 0X80;
+  char *buffer = NULL;
+
+  while (1) {
+    buffer = reallocWrapper(buffer, size<<=1);
+
+    {
+      int length = readlink("/proc/self/exe", buffer, size);
+
+      if (length == -1) {
+        if (errno != ENOENT) LogError("readlink");
+        break;
+      }
+
+      if (length < size) {
+        buffer[length] = 0;
+        path = strdupWrapper(buffer);
+        break;
+      }
+    }
+  }
+
+  free(buffer);
+  return path;
+}
+
 char *
 getBootParameters (const char *name) {
   size_t nameLength = strlen(name);
