@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -32,7 +32,7 @@ loadSharedObject (const char *path) {
 }
 
 void 
-unloadSharedObject (const void *object) {
+unloadSharedObject (void *object) {
 #ifdef HAVE_FUNC_DLOPEN 
   dlclose((void *)object);
 #endif /* HAVE_FUNC_DLOPEN */
