@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,21 +15,22 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
-#include <regex.h>
 #include <fcntl.h>
 #include <sys/stat.h>
 
+#ifdef HAVE_REGEX_H
+#include <regex.h>
+#endif /* HAVE_REGEX_H */
+
 #include "misc.h"
-#include "usb.h"
+#include "io_usb.h"
 #include "usb_internal.h"
 
 int
@@ -44,7 +45,7 @@ usbControlRead (
   int length,
   int timeout
 ) {
-  return usbControlTransfer(device, USB_DIRECTION_INPUT, recipient, type,
+  return usbControlTransfer(device, UsbControlDirection_Input, recipient, type,
                             request, value, index, buffer, length, timeout);
 }
 
@@ -60,7 +61,7 @@ usbControlWrite (
   int length,
   int timeout
 ) {
-  return usbControlTransfer(device, USB_DIRECTION_OUTPUT, recipient, type,
+  return usbControlTransfer(device, UsbControlDirection_Output, recipient, type,
                             request, value, index, (void *)buffer, length, timeout);
 }
 
@@ -73,11 +74,26 @@ usbGetDescriptor (
   UsbDescriptor *descriptor,
   int timeout
 ) {
-  return usbControlRead(device, USB_RECIPIENT_DEVICE, USB_TYPE_STANDARD,
-                        USB_REQ_GET_DESCRIPTOR, (type << 8) | number, index,
+  return usbControlRead(device, UsbControlRecipient_Device, UsbControlType_Standard,
+                        UsbStandardRequest_GetDescriptor, (type << 8) | number, index,
                         descriptor->bytes, sizeof(descriptor->bytes), timeout);
 }
 
+int
+usbGetDeviceDescriptor (
+  UsbDevice *device,
+  UsbDeviceDescriptor *descriptor
+) {
+  UsbDescriptor desc;
+  int size = usbGetDescriptor(device, UsbDescriptorType_Device, 0, 0, &desc, 1000);
+
+  if (size != -1) {
+    *descriptor = desc.device;
+  }
+
+  return size;
+}
+
 int
 usbGetLanguage (
   UsbDevice *device,
@@ -85,7 +101,7 @@ usbGetLanguage (
   int timeout
 ) {
   UsbDescriptor descriptor;
-  int size = usbGetDescriptor(device, USB_DESCRIPTOR_TYPE_STRING,
+  int size = usbGetDescriptor(device, UsbDescriptorType_String,
                               0, 0, &descriptor, timeout);
   if (size != -1) {
     if (size >= 4) {
@@ -110,7 +126,7 @@ usbDecodeString (const UsbStringDescriptor *descriptor) {
       string[count] = character;
     }
   } else {
-    LogError("USB string allocation");
+    LogError("USB string allocate");
   }
   return string;
 }
@@ -127,7 +143,7 @@ usbGetString (
     if (!usbGetLanguage(device, &device->language, timeout))
       return NULL;
 
-  if (usbGetDescriptor(device, USB_DESCRIPTOR_TYPE_STRING,
+  if (usbGetDescriptor(device, UsbDescriptorType_String,
                        number, device->language,
                        &descriptor, timeout) == -1)
     return NULL;
@@ -158,6 +174,8 @@ usbStringEquals (const char *reference, const char *value) {
 int
 usbStringMatches (const char *reference, const char *value) {
   int ok = 0;
+
+#ifdef HAVE_REGEX_H
   regex_t expression;
   if (regcomp(&expression, value, REG_EXTENDED|REG_NOSUB) == 0) {
     if (regexec(&expression, reference, 0, NULL, 0) == 0) {
@@ -165,6 +183,8 @@ usbStringMatches (const char *reference, const char *value) {
     }
     regfree(&expression);
   }
+#endif /* HAVE_REGEX_H */
+
   return ok;
 }
 
@@ -206,28 +226,43 @@ usbVerifySerialNumber (UsbDevice *device, const char *string) {
                          device->descriptor.iSerialNumber, string);
 }
 
+const UsbDeviceDescriptor *
+usbDeviceDescriptor (UsbDevice *device) {
+  return &device->descriptor;
+}
+
 int
 usbGetConfiguration (
   UsbDevice *device,
-  unsigned char *number
+  unsigned char *configuration
 ) {
-  int size = usbControlRead(device, USB_RECIPIENT_DEVICE, USB_TYPE_STANDARD,
-                            USB_REQ_GET_CONFIGURATION, 0, 0,
-                            number, sizeof(*number), 1000);
+  int size = usbControlRead(device, UsbControlRecipient_Device, UsbControlType_Standard,
+                            UsbStandardRequest_GetConfiguration, 0, 0,
+                            configuration, sizeof(*configuration), 1000);
   return size;
 }
 
-static int
-usbReadConfiguration (UsbDevice *device) {
-  if (device->configurationDescriptor) return 1;
+static void
+usbDeallocateConfigurationDescriptor (UsbDevice *device) {
+  if (device->configuration) {
+    free(device->configuration);
+    device->configuration = NULL;
+  }
+}
 
-  {
+const UsbConfigurationDescriptor *
+usbConfigurationDescriptor (
+  UsbDevice *device
+) {
+  if (!device->configuration) {
     unsigned char configuration;
+
     if (usbGetConfiguration(device, &configuration) != -1) {
       UsbDescriptor descriptor;
       unsigned char number;
+
       for (number=0; number<device->descriptor.bNumConfigurations; number++) {
-        int size = usbGetDescriptor(device, USB_DESCRIPTOR_TYPE_CONFIGURATION,
+        int size = usbGetDescriptor(device, UsbDescriptorType_Configuration,
                                     number, 0, &descriptor, 1000);
         if (size == -1) {
           LogPrint(LOG_WARNING, "USB configuration descriptor not readable: %d", number);
@@ -238,23 +273,104 @@ usbReadConfiguration (UsbDevice *device) {
 
       if (number < device->descriptor.bNumConfigurations) {
         int length = getLittleEndian(descriptor.configuration.wTotalLength);
-        UsbDescriptor *descriptors = malloc(length);
-        if (descriptors) {
-          int size = usbControlRead(device, USB_RECIPIENT_DEVICE, USB_TYPE_STANDARD,
-                                    USB_REQ_GET_DESCRIPTOR,
-                                    (USB_DESCRIPTOR_TYPE_CONFIGURATION << 8) | number,
-                                    0, descriptors, length, 1000);
-          if (size != -1) {
-            device->configurationDescriptor = descriptors;
-            device->configurationLength = length;
-            return 1;
+        UsbDescriptor *descriptors;
+
+        if ((descriptors = malloc(length))) {
+          int size;
+
+          if (length > sizeof(descriptor)) {
+            size = usbControlRead(device, UsbControlRecipient_Device, UsbControlType_Standard,
+                                  UsbStandardRequest_GetDescriptor,
+                                  (UsbDescriptorType_Configuration << 8) | number,
+                                  0, descriptors, length, 1000);
+          } else {
+            memcpy(descriptors, &descriptor, (size = length));
           }
-          free(descriptors);
+
+          if (size != -1) {
+            device->configuration = &descriptors->configuration;
+          } else {
+            free(descriptors);
+          }
+        } else {
+          LogError("USB configuration descriptor allocate");
         }
+      } else {
+        LogPrint(LOG_ERR, "USB configuration descriptor not found: %d", configuration);
       }
     }
   }
-  return 0;
+
+  return device->configuration;
+}
+
+int
+usbConfigureDevice (
+  UsbDevice *device,
+  unsigned char configuration
+) {
+  usbCloseInterface(device);
+  if (!usbSetConfiguration(device, configuration)) return 0;
+  usbDeallocateConfigurationDescriptor(device);
+  return 1;
+}
+
+int
+usbNextDescriptor (
+  UsbDevice *device,
+  const UsbDescriptor **descriptor
+) {
+  if (*descriptor) {
+    const UsbDescriptor *next = (UsbDescriptor *)&(*descriptor)->bytes[(*descriptor)->header.bLength];
+    const UsbDescriptor *first = (UsbDescriptor *)device->configuration;
+    unsigned int length = getLittleEndian(first->configuration.wTotalLength);
+    if ((&next->bytes[0] - &first->bytes[0]) >= length) return 0;
+    if ((&next->bytes[next->header.bLength] - &first->bytes[0]) > length) return 0;
+    *descriptor = next;
+  } else if (usbConfigurationDescriptor(device)) {
+    *descriptor = (UsbDescriptor *)device->configuration;
+  } else {
+    return 0;
+  }
+  return 1;
+}
+
+const UsbInterfaceDescriptor *
+usbInterfaceDescriptor (
+  UsbDevice *device,
+  unsigned char interface,
+  unsigned char alternative
+) {
+  const UsbDescriptor *descriptor = NULL;
+
+  while (usbNextDescriptor(device, &descriptor)) {
+    if (descriptor->interface.bDescriptorType == UsbDescriptorType_Interface)
+      if (descriptor->interface.bInterfaceNumber == interface)
+        if (descriptor->interface.bAlternateSetting == alternative)
+          return &descriptor->interface;
+  }
+
+  LogPrint(LOG_WARNING, "USB: interface descriptor not found: %d.%d", interface, alternative);
+  errno = ENOENT;
+  return NULL;
+}
+
+const UsbEndpointDescriptor *
+usbEndpointDescriptor (
+  UsbDevice *device,
+  unsigned char endpointAddress
+) {
+  const UsbDescriptor *descriptor = NULL;
+
+  while (usbNextDescriptor(device, &descriptor)) {
+    if (descriptor->endpoint.bDescriptorType == UsbDescriptorType_Endpoint)
+      if (descriptor->endpoint.bEndpointAddress == endpointAddress)
+        return &descriptor->endpoint;
+  }
+
+  LogPrint(LOG_WARNING, "USB: endpoint descriptor not found: %02X", endpointAddress);
+  errno = ENOENT;
+  return NULL;
 }
 
 static void
@@ -262,15 +378,25 @@ usbDeallocateEndpoint (void *item, void *data) {
   UsbEndpoint *endpoint = item;
 
   switch (USB_ENDPOINT_DIRECTION(endpoint->descriptor)) {
-    case USB_ENDPOINT_DIRECTION_INPUT:
-      if (endpoint->direction.input.pending)
+    case UsbEndpointDirection_Input:
+      if (endpoint->direction.input.pending) {
         deallocateQueue(endpoint->direction.input.pending);
-      if (endpoint->direction.input.completed)
+        endpoint->direction.input.pending = NULL;
+      }
+
+      if (endpoint->direction.input.completed) {
         free(endpoint->direction.input.completed);
+        endpoint->direction.input.completed = NULL;
+      }
+
       break;
   }
 
-  usbDeallocateEndpointExtension(endpoint);
+  if (endpoint->extension) {
+    usbDeallocateEndpointExtension(endpoint);
+    endpoint->extension = NULL;
+  }
+
   free(endpoint);
 }
 
@@ -283,82 +409,71 @@ usbTestEndpoint (void *item, void *data) {
 
 UsbEndpoint *
 usbGetEndpoint (UsbDevice *device, unsigned char endpointAddress) {
-  UsbEndpoint *endpoint = findItem(device->endpoints, usbTestEndpoint, &endpointAddress);
-  if (endpoint) return endpoint;
+  UsbEndpoint *endpoint;
+  const UsbEndpointDescriptor *descriptor;
 
-  if (usbReadConfiguration(device)) {
-    int found = 0;
-    int offset = 0;
-    while (offset < device->configurationLength) {
-      UsbEndpointDescriptor *descriptor = (UsbEndpointDescriptor *)((unsigned char *)device->configurationDescriptor + offset);
-      offset += descriptor->bLength;
-      if (descriptor->bDescriptorType != USB_DESCRIPTOR_TYPE_ENDPOINT) continue;
-      if (descriptor->bEndpointAddress != endpointAddress) continue;
-      found = 1;
+  if ((endpoint = findItem(device->endpoints, usbTestEndpoint, &endpointAddress))) return endpoint;
 
-      {
-        const char *direction;
-        const char *transfer;
+  if ((descriptor = usbEndpointDescriptor(device, endpointAddress))) {
+    {
+      const char *direction;
+      const char *transfer;
 
-        switch (USB_ENDPOINT_DIRECTION(descriptor)) {
-          default:                            direction = "?";   break;
-          case USB_ENDPOINT_DIRECTION_INPUT:  direction = "in";  break;
-          case USB_ENDPOINT_DIRECTION_OUTPUT: direction = "out"; break;
-        }
-
-        switch (USB_ENDPOINT_TRANSFER(descriptor)) {
-          default:                                transfer = "?";   break;
-          case USB_ENDPOINT_TRANSFER_CONTROL:     transfer = "ctl"; break;
-          case USB_ENDPOINT_TRANSFER_ISOCHRONOUS: transfer = "iso"; break;
-          case USB_ENDPOINT_TRANSFER_BULK:        transfer = "blk"; break;
-          case USB_ENDPOINT_TRANSFER_INTERRUPT:   transfer = "int"; break;
-        }
-
-        LogPrint(LOG_DEBUG, "USB: ept=%02X dir=%s xfr=%s pkt=%d ivl=%dms",
-                 descriptor->bEndpointAddress, direction, transfer,
-                 getLittleEndian(descriptor->wMaxPacketSize),
-                 descriptor->bInterval);
+      switch (USB_ENDPOINT_DIRECTION(descriptor)) {
+        default:                            direction = "?";   break;
+        case UsbEndpointDirection_Input:  direction = "in";  break;
+        case UsbEndpointDirection_Output: direction = "out"; break;
       }
 
-      if ((endpoint = malloc(sizeof(*endpoint)))) {
-        memset(endpoint, 0, sizeof(*endpoint));
-        endpoint->device = device;
-        endpoint->descriptor = descriptor;
-
-        switch (USB_ENDPOINT_DIRECTION(endpoint->descriptor)) {
-          case USB_ENDPOINT_DIRECTION_INPUT:
-            endpoint->direction.input.pending = NULL;
-            endpoint->direction.input.completed = NULL;
-            endpoint->direction.input.buffer = NULL;
-            endpoint->direction.input.length = 0;
-            break;
-        }
-
-        endpoint->extension = NULL;
-        if (usbAllocateEndpointExtension(endpoint)) {
-          if (enqueueItem(device->endpoints, endpoint)) return endpoint;
-          usbDeallocateEndpointExtension(endpoint);
-        }
-
-        free(endpoint);
+      switch (USB_ENDPOINT_TRANSFER(descriptor)) {
+        default:                                transfer = "?";   break;
+        case UsbEndpointTransfer_Control:     transfer = "ctl"; break;
+        case UsbEndpointTransfer_Isochronous: transfer = "iso"; break;
+        case UsbEndpointTransfer_Bulk:        transfer = "blk"; break;
+        case UsbEndpointTransfer_Interrupt:   transfer = "int"; break;
       }
-      break;
+
+      LogPrint(LOG_DEBUG, "USB: ept=%02X dir=%s xfr=%s pkt=%d ivl=%dms",
+               descriptor->bEndpointAddress, direction, transfer,
+               getLittleEndian(descriptor->wMaxPacketSize),
+               descriptor->bInterval);
+    }
+
+    if ((endpoint = malloc(sizeof(*endpoint)))) {
+      memset(endpoint, 0, sizeof(*endpoint));
+      endpoint->device = device;
+      endpoint->descriptor = descriptor;
+
+      switch (USB_ENDPOINT_DIRECTION(endpoint->descriptor)) {
+        case UsbEndpointDirection_Input:
+          endpoint->direction.input.pending = NULL;
+          endpoint->direction.input.completed = NULL;
+          endpoint->direction.input.buffer = NULL;
+          endpoint->direction.input.length = 0;
+          break;
+      }
+
+      endpoint->extension = NULL;
+      if (usbAllocateEndpointExtension(endpoint)) {
+        if (enqueueItem(device->endpoints, endpoint)) return endpoint;
+        usbDeallocateEndpointExtension(endpoint);
+      }
+
+      free(endpoint);
     }
-    if (!found) errno = ENOENT;
   }
 
-  LogPrint(LOG_WARNING, "USB: endpoint not found: %02X", endpointAddress);
   return NULL;
 }
 
 UsbEndpoint *
 usbGetInputEndpoint (UsbDevice *device, unsigned char endpointNumber) {
-  return usbGetEndpoint(device, endpointNumber|USB_ENDPOINT_DIRECTION_INPUT);
+  return usbGetEndpoint(device, endpointNumber|UsbEndpointDirection_Input);
 }
 
 UsbEndpoint *
 usbGetOutputEndpoint (UsbDevice *device, unsigned char endpointNumber) {
-  return usbGetEndpoint(device, endpointNumber|USB_ENDPOINT_DIRECTION_OUTPUT);
+  return usbGetEndpoint(device, endpointNumber|UsbEndpointDirection_Output);
 }
 
 static void
@@ -395,42 +510,84 @@ usbApplyInputFilters (UsbDevice *device, void *buffer, int size, int *length) {
   return 1;
 }
 
+void
+usbCloseInterface (
+  UsbDevice *device
+) {
+  if (device->endpoints) deleteElements(device->endpoints);
+
+  if (device->interface) {
+    usbReleaseInterface(device, device->interface->bInterfaceNumber);
+    device->interface = NULL;
+  }
+}
+
+int
+usbOpenInterface (
+  UsbDevice *device,
+  unsigned char interface,
+  unsigned char alternative
+) {
+  const UsbInterfaceDescriptor *descriptor = usbInterfaceDescriptor(device, interface, alternative);
+  if (!descriptor) return 0;
+  if (descriptor == device->interface) return 1;
+
+  if (device->interface)
+    if (device->interface->bInterfaceNumber != interface)
+      usbCloseInterface(device);
+
+  if (!device->interface)
+    if (!usbClaimInterface(device, interface))
+      return 0;
+
+  if (usbSetAlternative(device, interface, alternative)) {
+    device->interface = descriptor;
+    return 1;
+  }
+
+  if (!device->interface) usbReleaseInterface(device, interface);
+  return 0;
+}
+
 void
 usbCloseDevice (UsbDevice *device) {
-  usbDeallocateDeviceExtension(device);
-  deallocateQueue(device->inputFilters);
-  deallocateQueue(device->endpoints);
-  close(device->file);
-  free(device->path);
-  if (device->configurationDescriptor) free(device->configurationDescriptor);
+  usbCloseInterface(device);
+
+  if (device->endpoints) {
+    deallocateQueue(device->endpoints);
+    device->endpoints = NULL;
+  }
+
+  if (device->inputFilters) {
+    deallocateQueue(device->inputFilters);
+    device->inputFilters = NULL;
+  }
+
+  if (device->extension) {
+    usbDeallocateDeviceExtension(device);
+    device->extension = NULL;
+  }
+
+  usbDeallocateConfigurationDescriptor(device);
   free(device);
 }
 
-UsbDevice *
-usbOpenDevice (const char *path) {
+static UsbDevice *
+usbOpenDevice (void *extension) {
   UsbDevice *device;
   if ((device = malloc(sizeof(*device)))) {
     memset(device, 0, sizeof(*device));
+    device->extension = extension;
 
-    if ((device->path = strdup(path))) {
-      if ((device->endpoints = newQueue(usbDeallocateEndpoint, NULL))) {
-        if ((device->inputFilters = newQueue(usbDeallocateInputFilter, NULL))) {
-          if ((device->file = open(path, O_RDWR)) != -1) {
-            device->extension = NULL;
-            if (usbAllocateDeviceExtension(device)) {
-              if (usbReadDeviceDescriptor(device))
-                if (device->descriptor.bDescriptorType == USB_DESCRIPTOR_TYPE_DEVICE)
-                  if (device->descriptor.bLength == USB_DESCRIPTOR_SIZE_DEVICE)
-                    return device;
-              usbDeallocateDeviceExtension(device);
-            }
-            close(device->file);
-          }
-          deallocateQueue(device->inputFilters);
-        }
-        deallocateQueue(device->endpoints);
+    if ((device->endpoints = newQueue(usbDeallocateEndpoint, NULL))) {
+      if ((device->inputFilters = newQueue(usbDeallocateInputFilter, NULL))) {
+        if (usbReadDeviceDescriptor(device))
+          if (device->descriptor.bDescriptorType == UsbDescriptorType_Device)
+            if (device->descriptor.bLength == UsbDescriptorSize_Device)
+              return device;
+        deallocateQueue(device->inputFilters);
       }
-      free(device->path);
+      deallocateQueue(device->endpoints);
     }
     free(device);
   }
@@ -440,9 +597,9 @@ usbOpenDevice (const char *path) {
 }
 
 UsbDevice *
-usbTestDevice (const char *path, UsbDeviceChooser chooser, void *data) {
+usbTestDevice (void *extension, UsbDeviceChooser chooser, void *data) {
   UsbDevice *device;
-  if ((device = usbOpenDevice(path))) {
+  if ((device = usbOpenDevice(extension))) {
     LogPrint(LOG_DEBUG, "USB: testing: vendor=%04X product=%04X",
              device->descriptor.idVendor,
              device->descriptor.idProduct);
@@ -454,16 +611,12 @@ usbTestDevice (const char *path, UsbDeviceChooser chooser, void *data) {
     }
 
     errno = ENOENT;
+    device->extension = NULL;
     usbCloseDevice(device);
   }
   return NULL;
 }
 
-const UsbDeviceDescriptor *
-usbDeviceDescriptor (UsbDevice *device) {
-  return &device->descriptor;
-}
-
 static void
 usbDeallocatePendingInputRequest (void *item, void *data) {
   void *request = item;
@@ -518,49 +671,70 @@ usbAwaitInput (
   unsigned char endpointNumber,
   int timeout
 ) {
-  UsbEndpoint *endpoint = usbGetInputEndpoint(device, endpointNumber);
-  if (!endpoint) return 0;
+  UsbEndpoint *endpoint;
+  int interval;
+
+  if (!(endpoint = usbGetInputEndpoint(device, endpointNumber))) return 0;
   if (endpoint->direction.input.completed) return 1;
 
+  interval = endpoint->descriptor->bInterval;
+  interval = MAX(20, interval);
+
   if (!(endpoint->direction.input.pending && getQueueSize(endpoint->direction.input.pending))) {
     int size = getLittleEndian(endpoint->descriptor->wMaxPacketSize);
     unsigned char *buffer = malloc(size);
     if (buffer) {
       int count;
-      while ((count = usbReadEndpoint(device, endpointNumber, buffer, size, MAX(10, timeout))) != -1) {
-        if (count) {
+
+      if (timeout) hasTimedOut(0);
+      while (1) {
+        while ((count = usbReadEndpoint(device, endpointNumber, buffer, size,
+                                        MAX(interval, timeout))) != -1) {
+          if (!count) {
+            errno = EAGAIN;
+            break;
+          }
+
           endpoint->direction.input.buffer = buffer;
           endpoint->direction.input.length = count;
           endpoint->direction.input.completed = buffer;
           return 1;
         }
+#ifdef ETIMEDOUT
+        if (errno == ETIMEDOUT) errno = EAGAIN;
+#endif /* ETIMEDOUT */
+
+        if (errno != EAGAIN) break;
+        if (!timeout) break;
+        if (hasTimedOut(timeout)) break;
+        approximateDelay(interval);
       }
-      if (errno == ETIMEDOUT) errno = EAGAIN;
 
       free(buffer);
     }
     return 0;
   }
 
+  if (timeout) hasTimedOut(0);
   while (1) {
     UsbResponse response;
     void *request;
-    while (!(request = usbReapResponse(device, &response, 0))) {
-      if (errno != EAGAIN) return 0;
-      if (timeout <= 0) return 0;
 
-      {
-        const int interval = 10;
-        approximateDelay(interval);
-        timeout -= interval;
-      }
+    while (!(request = usbReapResponse(device,
+                                       endpointNumber | UsbEndpointDirection_Input,
+                                       &response, 0))) {
+      if (errno != EAGAIN) return 0;
+      if (!timeout) return 0;
+      if (hasTimedOut(timeout)) return 0;
+      approximateDelay(interval);
     }
+
     usbAddPendingInputRequest(endpoint);
     deleteItem(endpoint->direction.input.pending, request);
 
-    if (response.length) {
+    if (response.count > 0) {
       endpoint->direction.input.buffer = response.buffer;
-      endpoint->direction.input.length = response.length;
+      endpoint->direction.input.length = response.count;
       endpoint->direction.input.completed = request;
       return 1;
     }
@@ -610,7 +784,7 @@ usbReapInput (
 
 static int
 usbSetBelkinAttribute (UsbDevice *device, unsigned char request, int value) {
-  return usbControlWrite(device, USB_RECIPIENT_DEVICE, USB_TYPE_VENDOR,
+  return usbControlWrite(device, UsbControlRecipient_Device, UsbControlType_Vendor,
                          request, value, 0, NULL, 0, 1000) != -1;
 }
 static int
@@ -721,7 +895,7 @@ usbFtdiInputFilter (UsbInputFilterData *data) {
 }
 static int
 usbSetFtdiAttribute (UsbDevice *device, unsigned char request, int value, int index) {
-  return usbControlWrite(device, USB_RECIPIENT_DEVICE, USB_TYPE_VENDOR,
+  return usbControlWrite(device, UsbControlRecipient_Device, UsbControlType_Vendor,
                          request, value, index, NULL, 0, 1000) != -1;
 }
 static int
@@ -902,6 +1076,36 @@ usbGetSerialOperations (UsbDevice *device) {
       usbFtdiInputFilter
     }
     ,
+    { /* Baum FTDI: 24 cells  */
+      0X0403, 0XFE71,
+      &usbFtdiOperations_FT232BM,
+      usbFtdiInputFilter
+    }
+    ,
+    { /* Baum FTDI: 40 cells  */
+      0X0403, 0XFE72,
+      &usbFtdiOperations_FT232BM,
+      usbFtdiInputFilter
+    }
+    ,
+    { /* Baum FTDI: 32 cells  */
+      0X0403, 0XFE73,
+      &usbFtdiOperations_FT232BM,
+      usbFtdiInputFilter
+    }
+    ,
+    { /* Baum FTDI: 64 cells  */
+      0X0403, 0XFE74,
+      &usbFtdiOperations_FT232BM,
+      usbFtdiInputFilter
+    }
+    ,
+    { /* Baum FTDI: 80 cells  */
+      0X0403, 0XFE75,
+      &usbFtdiOperations_FT232BM,
+      usbFtdiInputFilter
+    }
+    ,
     {}
   };
   const UsbSerialAdapter *sa = usbSerialAdapters;
@@ -934,30 +1138,28 @@ usbChooseChannel (UsbDevice *device, void *data) {
     if (USB_IS_PRODUCT(descriptor, definition->vendor, definition->product)) {
       if (!usbVerifySerialNumber(device, choose->serialNumber)) break;
 
-      if (usbSetConfiguration(device, definition->configuration)) {
-        if (usbClaimInterface(device, definition->interface)) {
-          if (usbSetAlternative(device, definition->interface, definition->alternative)) {
-            int ok = 1;
+      if (usbConfigureDevice(device, definition->configuration)) {
+        if (usbOpenInterface(device, definition->interface, definition->alternative)) {
+          const UsbSerialOperations *serial = usbGetSerialOperations(device);
+          int ok = 1;
 
-            if (definition->baud) {
-              const UsbSerialOperations *serial = usbGetSerialOperations(device);
-              if (serial) {
-                if (!serial->setBaud(device, definition->baud)) ok = 0;
-                if (!serial->setFlowControl(device, definition->flowControl)) ok = 0;
-                if (!serial->setDataFormat(device, definition->dataBits, definition->stopBits, definition->parity)) ok = 0;
-              } else {
-                LogPrint(LOG_WARNING, "Unsupported serial adapter: vendor=%04X product=%04X",
-                         definition->vendor, definition->product);
-                ok = 0;
-              }
-            }
-
-            if (ok) {
-              choose->definition = definition;
-              return 1;
+          if (definition->baud) {
+            if (serial) {
+              if (!serial->setBaud(device, definition->baud)) ok = 0;
+              if (!serial->setFlowControl(device, definition->flowControl)) ok = 0;
+              if (!serial->setDataFormat(device, definition->dataBits, definition->stopBits, definition->parity)) ok = 0;
+            } else {
+              LogPrint(LOG_WARNING, "Unsupported serial adapter: vendor=%04X product=%04X",
+                       definition->vendor, definition->product);
+              ok = 0;
             }
           }
-          usbReleaseInterface(device, definition->interface);
+
+          if (ok) {
+            choose->definition = definition;
+            return 1;
+          }
+          usbCloseInterface(device);
         }
       }
     }
@@ -994,7 +1196,6 @@ usbFindChannel (const UsbChannelDefinition *definitions, const char *device) {
 
 void
 usbCloseChannel (UsbChannel *channel) {
-  usbReleaseInterface(channel->device, channel->definition.interface);
   usbCloseDevice(channel->device);
   free(channel);
 }
